import math
from pandas import read_excel, concat, DataFrame
from openpyxl import load_workbook
import csv
import numpy as np
from ..Analise.Movements import EyeMovement
from ..Analise.GeoMetrics import GeoMetrics
from ..Analise.SecondaryMetrics import SecondaryMetrics

class ParsedData:
    def __init__(self):
        self.dots = 0
        self.raw_data = None
        self.secondary = None
        self.time_frame = 0
        self.minSpeed = 0
        self.avrSpeed = 0
        self.maxSpeed = 0
        self.fixations = []
        self.saccades = []
        self.flag = []
        self.pong = 0
        self.false_fix = 0
        self.method = 0
        self.args = 200
        self.center_radius = 200

    def compare(self, other):
        l = min(len(self.flag), len(other.flag))
        fp = 0
        fn = 0
        tp = 0
        for i in range(0, l):
            if self.flag[i] == 's' and other.flag[i] != 's':
                fp += 1.0
            if other.flag[i] == 's':
                if self.flag[i] == other.flag[i]:
                    tp += 1.0
                else:
                    fn += 1.0
        return 2 * tp / (2 * tp + fp + fn)

    # вынимаем метрики из отрезка данных
    def parse(self, input_section, method=0, args=200, center_radius=2.5):
        print('parse...')
        self.raw_data = input_section
        self.center_radius = center_radius
        self.dots = len(input_section.positionData)
        self.sep_geo(input_section.FOV)
        self.time_frame = input_section.time_frame()
        self.minSpeed = min(input_section.velocityData)
        self.maxSpeed = max(input_section.velocityData)
        self.avrSpeed = sum(input_section.distanceData) / self.time_frame
        self.method = method
        self.args = args
        print('marking fixations...')
        self.flag = []
        if self.method != 2 and self.method != 1:
            self.method = 0
        if self.method == 2:
            self.get_fixation_by_abs_dist(args)
        if self.method == 1:
            self.get_fixation_by_area(args)
        if self.method == 0:
            self.get_fixation_by_speed_threshold(args)
        print('cleaning...')
        self.clean_fix()
        print('Pong = ' + self.pong.__str__() + ' False Fix = ' + self.false_fix.__str__())
        print('composing...')
        self.compose()

    def parse_pupil(self, input_section, file):
        print('parse by pupil...')
        self.raw_data = input_section
        self.center_radius = 0
        self.dots = len(input_section.positionData)
        self.time_frame = input_section.time_frame()
        self.minSpeed = min(input_section.velocityData)
        self.maxSpeed = max(input_section.velocityData)
        self.avrSpeed = sum(input_section.distanceData) / self.time_frame
        self.method = 4
        self.args = file
        print('marking fixations...')
        self.flag = []
        self.get_fixation_by_pupil(file)
        print('cleaning...')
        self.clean_fix()
        print('Pong = ' + self.pong.__str__() + ' False Fix = ' + self.false_fix.__str__())
        print('composing...')
        self.compose()

    def get_method(self):
        if self.method == 4:
            return 'by pupil_labs file '
        if self.method == 2:
            return 'abs dist ' + self.args.__str__()
        if self.method == 1:
            return 'area size ' + self.args.__str__()
        self.method = 0
        return 'speed ' + self.args.__str__()
    
    # Выделение нормированного X и Y 
    def sep_geo(self, FOV):  
        self.dots_X = []
        self.dots_Y = []
        for i in range(0, len(self.raw_data.positionData)):
            self.dots_X.append(self.raw_data.positionData[i].x / FOV)
            self.dots_Y.append(self.raw_data.positionData[i].y / FOV)

    # Алгоритм 1. Определение стратегий по скорости движения глаз между любыми двумя последовательными точками,
    # (Shic, Scassellati, & Chawarska, 2008).

    def get_fixation_by_speed_threshold(self, max_speed):
        print('max_speed ' + max_speed.__str__())
        self.flag.append('f')
        for i in range(0, self.dots - 1):
            if self.raw_data.velocityData[i] > max_speed:
                self.flag.append('s')
            else:
                self.flag.append('f')
    
    
    # Алгоритм 2. Определение стратегий по расстоянию между точками и центром фиксации, т.е. её радиусу.
    # (Camilli, Terenzi, & Nocera, 2008)

    def get_fixation_by_area(self, area_size):
        fix_area = self.raw_data.positionData[0]
        self.flag.append('f')
        for i in range(1, self.dots - 1):
            if self.flag[-1] == 'f':
                if fix_area.get_distance(self.raw_data.positionData[i]) > (area_size / 2):
                    self.flag.append('s')
                else:
                    self.flag.append('f')
            else:
                if self.raw_data.positionData[i - 1].get_distance(self.raw_data.positionData[i]) > (area_size / 2):
                    self.flag.append('s')
                else:
                    self.flag.append('f')
                    fix_area = self.raw_data.positionData[i - 1]

    # Алгоритм 3. Определение стратегий по максимальному расстоянию по горизонтали и вертикали,
    # покрываемому позициями взгляда при фиксации
    # Algorithm 3 Determination of strategies based on the maximum horizontal and vertical distance,
    # covered by the gaze positions inside fixation
    # (Salvucci & Goldberg). , 2000).

    def get_fixation_by_abs_dist(self, dist):
        max_x = min_x = self.raw_data.positionData[0].x
        max_y = min_y = self.raw_data.positionData[0].y
        self.flag.append('f')
        for i in range(1, self.dots - 1):
            max_x = max(max_x, self.raw_data.positionData[i].x)
            min_x = min(min_x, self.raw_data.positionData[i].x)
            max_y = max(max_y, self.raw_data.positionData[i].y)
            min_y = min(min_y, self.raw_data.positionData[i].y)
            if math.sqrt(pow(max_x - min_x, 2) + pow(max_y - min_y, 2)) > dist: # проверка порога / treshhold check
                self.flag.append('s')
                max_x = min_x = self.raw_data.positionData[i].x
                max_y = min_y = self.raw_data.positionData[i].y
            else:
                self.flag.append('f')

    def get_fixation_by_pupil(self, file):
        fix = []
        with open(file, newline='') as f:
            reader = csv.reader(f)
            head = True
            start_time = 0
            for row in reader:
                if head:
                    head = False
                    continue
                if start_time == 0:
                    start_time = float(row[3]) / 1000000000
                fix.append([float(row[3]) / 1000000000 - start_time, float(row[4]) / 1000000000 - start_time])

        print("Pupil time" + (fix[-1][1] - fix[0][0]).__str__())
        print("Frame time" + self.time_frame.__str__())
        delta_time = abs(self.time_frame - (fix[-1][1] - fix[0][0]))
        #start_time -= delta_time
        fix_i = 0
        while self.raw_data.positionData[0].time > fix[fix_i + 1][0]:
            fix_i += 1
        print(fix_i.__str__() + '  ' + self.raw_data.positionData[0].time.__str__() + '  '
              + fix[fix_i][0].__str__(), fix[fix_i][1].__str__())

        # print(fix)

        i = 0
        while i < self.dots:
            if fix_i >= len(fix):
                print('sss')
                break
            if self.raw_data.positionData[i].time < fix[fix_i][0]:
                self.flag.append('s')
            elif self.raw_data.positionData[i].time < fix[fix_i][1]:
                self.flag.append('f')
            else:
                fix_i += 1
                continue
            i += 1

    def clean_fix(self):
        for i in range(1, len(self.flag) - 2):
            if self.flag[i] == 's' and self.flag[i - 1] == 'f' and self.flag[i + 1] == 'f':
                self.pong += 1
                self.flag[i] = 'f'
        for i in range(1, len(self.flag) - 2):
            if self.flag[i] == 'f' and self.flag[i - 1] == 's' and self.flag[i + 1] == 's':
                self.false_fix += 1
                self.flag[i] = 's'

    def compose(self):
        fl = self.flag[0]
        self.new_movement(fl, self.raw_data.positionData[0])
        for i in range(1, len(self.flag) - 1):
            if fl == 's':
                self.saccades[-1].add_point(self.raw_data.positionData[i])
            else:
                if fl == self.flag[i]:
                    self.fixations[-1].add_point(self.raw_data.positionData[i])
            if fl != self.flag[i]:
                fl = self.flag[i]
                if fl == 's':
                    self.new_movement(fl, self.raw_data.positionData[i-1])
                    self.saccades[-1].add_point(self.raw_data.positionData[i])
                else:
                    self.new_movement(fl, self.raw_data.positionData[i])


    def new_movement(self, fl, point):
        if fl == 's':
            self.saccades.append(EyeMovement('s'))
            self.saccades[-1].add_point(point)
        else:
            self.fixations.append(EyeMovement('f'))
            self.fixations[-1].add_point(point)

    def calc_metrics(self):
        self.secondary = SecondaryMetrics()
        self.geo = GeoMetrics()

        # -----------------#
        x = np.array(self.dots_X)
        self.geo.x_mean = np.mean(x).reshape(1, 1)[0][0]
        self.geo.x_std = np.std(x).reshape(1, 1)[0][0]
        self.geo.x_min = np.min(x).reshape(1, 1)[0][0]
        self.geo.x_max = np.max(x).reshape(1, 1)[0][0]
        self.geo.x_25 = np.percentile(x, 25).reshape(1, 1)[0][0]
        self.geo.x_50 = np.percentile(x, 50).reshape(1, 1)[0][0]
        self.geo.x_75 = np.percentile(x, 75).reshape(1, 1)[0][0]
        # -----------------#
        y = np.array(self.dots_Y)
        self.geo.y_mean = np.mean(y).reshape(1, 1)[0][0]
        self.geo.y_std = np.std(y).reshape(1, 1)[0][0]
        self.geo.y_min = np.min(y).reshape(1, 1)[0][0]
        self.geo.y_max = np.max(y).reshape(1, 1)[0][0]
        self.geo.y_25 = np.percentile(y, 25).reshape(1, 1)[0][0]
        self.geo.y_50 = np.percentile(y, 50).reshape(1, 1)[0][0]
        self.geo.y_75 = np.percentile(y, 75).reshape(1, 1)[0][0]
        # -----------------#

        print('computing metrics...')
        print('fixations...')

        for fix in self.fixations:
            if len(fix.points) < 3:
                continue
            self.secondary.fix_time += fix.time
            self.secondary.fix_distance += fix.get_distance()
            self.secondary.max_f_speed = max(self.secondary.max_f_speed, fix.get_avr_speed())
            self.secondary.min_f_speed = min(self.secondary.min_f_speed, fix.get_avr_speed())
            # self.secondary.min_curve = min(self.secondary.min_curve, fix.get_curve())
            # self.secondary.max_curve = min(self.secondary.max_curve, fix.get_curve())
            # self.secondary.avr_curve += fix.get_curve()
            # spd = S / T = (S1 + S2 + S3) / (T1 + T2 + T3)
            if fix.time < 0.80:
                self.secondary.fix_l_80 += 1
            if fix.time > 0.180:
                self.secondary.fix_g_180 += 1
                self.secondary.fix_g_180_time += fix.time
                self.secondary.fix_g_180_count += 1
            if fix.time < 0.180:
                self.secondary.fix_l_180 += 1
                self.secondary.fix_l_180_time += fix.time
                self.secondary.fix_l_180_count += 1 
            if fix.time > 1:
                self.secondary.fix_g_1000 += 1
            if fix.time < 0.150:
                self.secondary.short_fix_time += fix.time
                self.secondary.short_fix_count += 1
                self.secondary.fix_l_150_dist += fix.get_distance()
                self.secondary.max_sf_speed = max(self.secondary.max_sf_speed, fix.get_avr_speed())
                self.secondary.min_sf_speed = min(self.secondary.min_sf_speed, fix.get_avr_speed())
            if 0.150 <= fix.time <= 0.900:
                self.secondary.med_fix_time += fix.time
                self.secondary.med_fix_count += 1
                self.secondary.fix_g_150_dist += fix.get_distance()
                self.secondary.max_lf_speed = max(self.secondary.max_lf_speed, fix.get_avr_speed())
                self.secondary.min_lf_speed = min(self.secondary.min_lf_speed, fix.get_avr_speed())
            if fix.time > 0.900:
                self.secondary.long_fix_time += fix.time
                self.secondary.long_fix_count += 1
                self.secondary.fix_g_150_dist += fix.get_distance()
                self.secondary.max_lf_speed = max(self.secondary.max_lf_speed, fix.get_avr_speed())
                self.secondary.min_lf_speed = min(self.secondary.min_lf_speed, fix.get_avr_speed())

        print('saccades...')
        for sacc in self.saccades:
            if len(sacc.points) < 3:
                continue
            self.secondary.sacc_time += sacc.time
            self.secondary.sacc_distance += sacc.get_distance()
            self.secondary.max_s_speed = max(self.secondary.max_s_speed, sacc.get_avr_speed())
            self.secondary.min_s_speed = min(self.secondary.min_s_speed, sacc.get_avr_speed())
            self.secondary.max_curve = max(self.secondary.max_curve, sacc.get_curve())
            self.secondary.min_curve = min(self.secondary.min_curve, sacc.get_curve())
            self.secondary.avr_curve += sacc.get_curve()
            self.secondary.max_s_length = max(self.secondary.max_s_length, sacc.get_distance())
            self.secondary.min_s_length = min(self.secondary.min_s_length, sacc.get_distance())
            self.secondary.max_s_time = max(self.secondary.max_s_time, sacc.time)
            self.secondary.min_s_time = min(self.secondary.min_s_time, sacc.time)
            self.secondary.sacc_length += sacc.get_distance()
            if sacc.get_abs_distance() > 6:
                self.secondary.long_sacc += 1
            else:
                self.secondary.short_sacc += 1

        i_count = 0
        s0 = self.raw_data.positionData[0]
        t0 = s0.time
        v0 = 0
        prev = s0
        dist = 0
        v = -1
        p = None

        for step in self.raw_data.positionData:
            if p is not None:
                if v != -1:
                    acc = abs(v - p.get_distance(step) / (step.time - p.time))  # / (step.time - p.time)
                    self.secondary.avr_acc += acc
                    self.secondary.min_acc = min(self.secondary.min_i_acc, acc / (step.time - p.time))
                    self.secondary.max_acc = max(self.secondary.max_i_acc, acc / (step.time - p.time))
                v = p.get_distance(step) / (step.time - p.time)
            p = step

            if step.time - t0 > 1:
                dt = step.time - t0
                dv = abs(v0 - dist/dt)
                self.secondary.avr_i_speed += dist / dt
                self.secondary.min_i_speed = min(self.secondary.min_i_speed, dist / dt)
                self.secondary.max_i_speed = max(self.secondary.max_i_speed, dist / dt)
                self.secondary.avr_i_acc += dv / dt
                self.secondary.min_i_acc = min(self.secondary.min_i_acc, dv / dt)
                self.secondary.max_i_acc = max(self.secondary.max_i_acc, dv / dt)
                i_count += 1
                s0 = step
                t0 = s0.time
                v0 = dist / dt
                dist = 0
            dist += step.get_distance(prev)
            prev = step


        self.secondary.avr_acc /= self.raw_data.positionData[-1].time - self.raw_data.positionData[2].time
        self.secondary.avr_i_speed /= i_count
        self.secondary.avr_i_acc /= i_count

        i_count = 0
        t0 = self.raw_data.positionData[0].time
        fix_count = 0
        for i in range(1, len(self.flag)):
            if self.flag[i] == 'f' and self.flag[i-1] == 's':
                fix_count += 1
            if self.raw_data.positionData[i].time - t0 > 1:
                freq = fix_count / (self.raw_data.positionData[i].time - t0)
                self.secondary.avr_freq += freq
                self.secondary.min_freq = min(self.secondary.min_freq, freq)
                self.secondary.max_freq = max(self.secondary.max_freq, freq)
                i_count += 1
                t0 = self.raw_data.positionData[i].time
                fix_count = 0
        self.secondary.avr_freq /= i_count
        print('calculating complete')
#   35/39
    #   35/39

    def get_row(self, file_name):
        print('forming a row... ' )
        
        return self.get_df_row(file_name).loc[0, :].values.flatten().tolist()

    def to_csv(self, file_name, file_to):
        print('csv file... ' + file_to)
        row = self.get_row(file_name)
        print('appending a row...')
        with open(file_to, 'a', newline='') as csvfile:
            _writer = csv.writer(csvfile, delimiter=';')
            _writer.writerow(row)
        print('writen')

    def get_df_row(self, file='none'):
        all_fix = self.secondary.short_fix_count + self.secondary.med_fix_count + self.secondary.long_fix_count

        df_row = DataFrame({'File' : [file],
                            'x_mean' : [self.geo.x_mean],
                            'x_std' : [self.geo.x_std],
                            'x_min' : [self.geo.x_min],
                            'x_max' : [self.geo.x_max],
                            'x_25' : [self.geo.x_25],
                            'x_50' : [self.geo.x_50],
                            'x_75' : [self.geo.x_75],
                            'y_mean' : [self.geo.y_mean],
                            'y_std' : [self.geo.y_std],
                            'y_min' : [self.geo.y_min],
                            'y_max' : [self.geo.y_max],
                            'y_25' : [self.geo.y_25],
                            'y_50' : [self.geo.y_50],
                            'y_75' : [self.geo.y_75],
                            'False Fixation, per minute': [self.false_fix / self.time_frame * 60],
                            'False Saccades, per minute': [self.pong / self.time_frame * 60],
                            'Saccades with amplitude > 6 degrees, per minute': [
                                self.secondary.long_sacc / self.time_frame * 60],
                            'Saccades with amplitude < 6 degrees, per minute': [
                                self.secondary.short_sacc / self.time_frame * 60],
                            'Max Curve': [self.secondary.max_curve],
                            'Average Curve': [self.secondary.avr_curve / (len(self.fixations) + len(self.saccades))],
                            'Min Curve': [self.secondary.min_curve],
                            'Time Frame': [self.time_frame],
                            'Saccades time': [self.secondary.sacc_time],
                            'Fixation time < 150 ms': [self.secondary.short_fix_time],
                            'Fixation time > 150 ms': [self.secondary.med_fix_time + self.secondary.long_fix_time],
                            'Fixation time between 150 and 900 ms': [self.secondary.med_fix_time],
                            'Fixation time > 900 ms': [self.secondary.long_fix_time],
                            'Fixation time < 180 ms': [self.secondary.fix_l_180_time],
                            'Fixation time > 180 ms': [self.secondary.fix_g_180_time],
                            '% of Fixations < 150 ms': [self.secondary.short_fix_count / all_fix * 100 if all_fix > 0 else -1],
                            '% of Fixations > 150 ms': [(1 - self.secondary.short_fix_count / all_fix) * 100 if all_fix > 0 else -1],
                            '% of Fixations between 150 and 900 ms': [self.secondary.med_fix_count / all_fix * 100 if all_fix > 0 else -1],
                            '% of Fixations > 900 ms': [self.secondary.long_fix_count / all_fix * 100 if all_fix > 0 else -1],
                            '% of Fixations < 180 ms': [self.secondary.fix_l_180 / all_fix * 100 if all_fix > 0 else -1],
                            '% of Fixations > 180 ms': [self.secondary.fix_g_180 / all_fix * 100 if all_fix > 0 else -1],
                            'Fixation time < 150 ms, per time': [self.secondary.short_fix_time / self.time_frame],
                            'Fixation time > 150 ms, per time': [
                                (self.secondary.med_fix_time + self.secondary.long_fix_time) / self.time_frame],
                            'Fixation time between 150 and 900 ms, per time': [
                                self.secondary.med_fix_time / self.time_frame],
                            'Fixation time > 900 ms, per time': [self.secondary.long_fix_time / self.time_frame],
                            'Fixation time < 180 ms, per time': [self.secondary.fix_l_180_time / self.time_frame],
                            'Fixation time > 180 ms, per time': [self.secondary.fix_g_180_time / self.time_frame],
                            '% of Fixations < 150 ms, per minute': [
                                self.secondary.short_fix_count / self.time_frame * 60],
                            '% of Fixations > 150 ms, per minute': [
                                (self.secondary.med_fix_count + self.secondary.long_fix_count) / self.time_frame * 60],
                            '% of Fixations between 150 and 900 ms, per minute': [
                                self.secondary.med_fix_count / self.time_frame * 60],
                            '% of Fixations > 900 ms, per minute': [
                                self.secondary.long_fix_count / self.time_frame * 60],
                            '% of Fixations < 180 ms, per minute': [self.secondary.fix_l_180 / self.time_frame * 60],
                            '% of Fixations > 180 ms, per minute': [self.secondary.fix_g_180 / self.time_frame * 60],
                            'Fixation Frequency': [len(self.fixations) / self.time_frame],
                            'Average Fix Frequency in interval (1s)': [self.secondary.avr_freq],
                            'Max Fix Frequency in interval (1s)': [self.secondary.max_freq],
                            'Average Acceleration': [self.secondary.avr_acc],
                            'Min Acceleration': [self.secondary.min_acc],
                            'Max Acceleration': [self.secondary.max_acc],
                            'Average Speed': [self.avrSpeed],
                            'Min Speed': [self.minSpeed],
                            'Max Speed': [self.maxSpeed],
                            'Average Acceleration in interval (1s)': [self.secondary.avr_i_acc],
                            'Min Acceleration in interval (1s)': [self.secondary.min_i_acc],
                            'Max Acceleration in interval (1s)': [self.secondary.max_i_acc],
                            'Average Speed in interval (1s)': [self.secondary.avr_i_speed],
                            'Min Speed in interval (1s)': [self.secondary.min_i_speed],
                            'Max Speed in interval (1s)': [self.secondary.max_i_speed],
                            'Average Fixation Speed': [self.secondary.fix_distance / self.secondary.fix_time if self.secondary.fix_time > 0 else - 1],
                            'Min Fixation Speed': [self.secondary.min_f_speed],
                            'Max Fixation Speed': [self.secondary.max_f_speed],
                            'Average Fixation Speed, < 150ms': [self.secondary.fix_l_150_dist / self.secondary.short_fix_time if self.secondary.short_fix_time > 0 else - 1],
                            'Min Fixation Speed, < 150ms': [self.secondary.min_sf_speed],
                            'Max Fixation Speed, < 150ms': [self.secondary.max_sf_speed],
                            'Average Fixation Speed, > 150ms': [self.secondary.fix_g_150_dist / (self.secondary.med_fix_time + self.secondary.long_fix_time) if (self.secondary.med_fix_time + self.secondary.long_fix_time) != 0 else -1],
                            'Min Fixation Speed, > 150ms': [self.secondary.min_lf_speed],
                            'Max Fixation Speed, > 150ms': [self.secondary.max_lf_speed],
                            'Average Saccade Speed': [self.secondary.sacc_distance / self.secondary.sacc_time if self.secondary.sacc_time > 0 else - 1],
                            'Min Saccade Speed': [self.secondary.min_s_speed],
                            'Max Saccade Speed': [self.secondary.max_s_speed],
                            'Average Saccade Length': [self.secondary.sacc_distance / len(self.saccades) if len(self.saccades) > 0 else - 1],
                            'Min Saccade Length': [self.secondary.min_s_length],
                            'Max Saccade Length': [self.secondary.max_s_length],
                            'Average Saccade Time': [self.secondary.sacc_time / len(self.saccades) if len(self.saccades) > 0 else - 1],
                            'Min Saccade Time': [self.secondary.min_s_time],
                            'Max Saccade Time': [self.secondary.max_s_time]
                            })
        return df_row

    def to_xls_by_row(self, file_name, file_to):
        print('excel reading file ' + file_to)
        print('excel file preparing ' + file_to)
        print('forming headline...')

        df = read_excel(file_to, engine='openpyxl')
        print(df.__str__())
        print('computing metrics...')
        print('forming a row...')
        #   35/39

        df_row = self.get_df_row(file_name)

        print('appending a row...')
        df = concat([df, df_row], ignore_index=True)
        print(df.__str__())
        print('writing...')
        update_spreadsheet(file_to, df, df_row, sheet_name='Sheet1')
        print('writen')

    def to_xls(self, file_name, file_to):
        print('Not effective')
        return
        print('excel reading file ' + file_to)
        print('excel file preparing ' + file_to)
        print('forming headline...')

        df = read_excel(file_to, engine='openpyxl')
        print(df.__str__())

        all_fix = self.secondary.short_fix_count + self.secondary.med_fix_count + self.secondary.long_fix_count
        # print(all_fix)

        new_row = len(df.get('File')) + 1
        update_cell(file_to, new_row, key_index(df, 'File'), file_name)
        update_cell(file_to, new_row, key_index(df, 'Method'), self.get_method())

        update_cell(file_to, new_row, key_index(df, 'False Fixation, per minute'),
                    self.false_fix / self.time_frame * 60)
        update_cell(file_to, new_row, key_index(df, 'False Saccades, per minute'),
                    self.pong / self.time_frame * 60)

        update_cell(file_to, new_row, key_index(df, 'Saccades with amplitude > 6 degrees, per minute'),
                    self.secondary.long_sacc / self.time_frame * 60)
        update_cell(file_to, new_row, key_index(df, 'Saccades with amplitude < 6 degrees, per minute'),
                    self.secondary.short_sacc / self.time_frame * 60)

        update_cell(file_to, new_row, key_index(df, 'Max Curve'), self.secondary.max_curve)
        update_cell(file_to, new_row, key_index(df, 'Average Curve'),
                    self.secondary.avr_curve / (len(self.fixations) + len(self.saccades)))
        update_cell(file_to, new_row, key_index(df, 'Min Curve'), self.secondary.min_curve)

        update_cell(file_to, new_row, key_index(df, 'Time Frame'), self.time_frame)
        update_cell(file_to, new_row, key_index(df, 'Saccades time'), self.secondary.sacc_time)

        update_cell(file_to, new_row, key_index(df, 'Fixation time < 150 ms'),
                    self.secondary.short_fix_time)
        update_cell(file_to, new_row, key_index(df, 'Fixation time > 150 ms'),
                    self.secondary.med_fix_time + self.secondary.long_fix_time)

        update_cell(file_to, new_row, key_index(df, 'Fixation time between 150 and 900 ms'),
                    self.secondary.med_fix_time)
        update_cell(file_to, new_row, key_index(df, 'Fixation time > 900 ms'),
                    self.secondary.long_fix_time)

        update_cell(file_to, new_row, key_index(df, 'Fixation time < 180 ms'),
                    self.secondary.fix_l_180_time)
        update_cell(file_to, new_row, key_index(df, 'Fixation time > 180 ms'),
                    self.secondary.fix_g_180_time)

        update_cell(file_to, new_row, key_index(df, '% of Fixations < 150 ms'),
                    self.secondary.short_fix_count / all_fix * 100)
        update_cell(file_to, new_row, key_index(df, '% of Fixations > 150 ms'),
                    (1 - self.secondary.short_fix_count / all_fix) * 100)

        update_cell(file_to, new_row, key_index(df, '% of Fixations between 150 and 900 ms'),
                    self.secondary.med_fix_count / all_fix * 100)

        update_cell(file_to, new_row, key_index(df, '% of Fixations > 900 ms'),
                    self.secondary.long_fix_count / all_fix * 100)

        update_cell(file_to, new_row, key_index(df, '% of Fixations < 180 ms'),
                    self.secondary.fix_l_180 / all_fix * 100)

        update_cell(file_to, new_row, key_index(df, '% of Fixations > 180 ms'),
                    self.secondary.fix_g_180 / all_fix * 100)

        # , per minute

        update_cell(file_to, new_row, key_index(df, 'Fixation time < 150 ms, per time'),
                    self.secondary.short_fix_time / self.time_frame)
        update_cell(file_to, new_row, key_index(df, 'Fixation time > 150 ms, per time'),
                    (self.secondary.med_fix_time + self.secondary.long_fix_time) / self.time_frame)
        update_cell(file_to, new_row, key_index(df, 'Fixation time between 150 and 900 ms, per time'),
                    self.secondary.med_fix_time / self.time_frame)
        update_cell(file_to, new_row, key_index(df, 'Fixation time > 900 ms, per time'),
                    self.secondary.long_fix_time / self.time_frame)

        update_cell(file_to, new_row, key_index(df, 'Fixation time < 180 ms, per time'),
                    self.secondary.fix_l_180_time / self.time_frame)
        update_cell(file_to, new_row, key_index(df, 'Fixation time > 180 ms, per time'),
                    self.secondary.fix_g_180_time / self.time_frame)

        update_cell(file_to, new_row, key_index(df, 'Fixations < 150 ms, per minute'),
                    self.secondary.short_fix_count / self.time_frame * 60)
        update_cell(file_to, new_row, key_index(df, 'Fixations > 150 ms, per minute'),
                    (self.secondary.med_fix_count + self.secondary.long_fix_count) / self.time_frame * 60)
        update_cell(file_to, new_row, key_index(df, 'Fixations between 150 and 900 ms, per minute'),
                    self.secondary.med_fix_count / self.time_frame * 60)
        update_cell(file_to, new_row, key_index(df, 'Fixations > 900 ms, per minute'),
                    self.secondary.long_fix_count / self.time_frame * 60)

        update_cell(file_to, new_row, key_index(df, 'Fixations < 180 ms, per minute'),
                    self.secondary.fix_l_180 / self.time_frame * 60)
        update_cell(file_to, new_row, key_index(df, 'Fixations > 180 ms, per minute'),
                    self.secondary.fix_g_180 / self.time_frame * 60)

        update_cell(file_to, new_row, key_index(df, 'Fixation Frequency'), len(self.fixations) / self.time_frame)
        update_cell(file_to, new_row, key_index(df, 'Average Fix Frequency in interval (1s)'), self.secondary.avr_freq)
        update_cell(file_to, new_row, key_index(df, 'Max Fix Frequency in interval (1s)'), self.secondary.max_freq)

        update_cell(file_to, new_row, key_index(df, 'Average Acceleration'), self.secondary.avr_acc)
        update_cell(file_to, new_row, key_index(df, 'Min Acceleration'), self.secondary.min_acc)
        update_cell(file_to, new_row, key_index(df, 'Max Acceleration'), self.secondary.max_acc)

        update_cell(file_to, new_row, key_index(df, 'Average Speed'), self.avrSpeed)
        update_cell(file_to, new_row, key_index(df, 'Min Speed'), self.minSpeed)
        update_cell(file_to, new_row, key_index(df, 'Max Speed'), self.maxSpeed)

        update_cell(file_to, new_row, key_index(df, 'Average Acceleration in interval (1s)'), self.secondary.avr_i_acc)
        update_cell(file_to, new_row, key_index(df, 'Min Acceleration in interval (1s)'), self.secondary.min_i_acc)
        update_cell(file_to, new_row, key_index(df, 'Max Acceleration in interval (1s)'), self.secondary.max_i_acc)

        update_cell(file_to, new_row, key_index(df, 'Average Speed in interval (1s)'), self.secondary.avr_i_speed)
        update_cell(file_to, new_row, key_index(df, 'Min Speed in interval (1s)'), self.secondary.min_i_speed)
        update_cell(file_to, new_row, key_index(df, 'Max Speed in interval (1s)'), self.secondary.max_i_speed)

        update_cell(file_to, new_row, key_index(df, 'Average Fixation Speed'),
                    self.secondary.fix_distance / self.secondary.fix_time)
        update_cell(file_to, new_row, key_index(df, 'Min Fixation Speed'), self.secondary.min_f_speed)
        update_cell(file_to, new_row, key_index(df, 'Max Fixation Speed'), self.secondary.max_f_speed)
        
        update_cell(file_to, new_row, key_index(df, 'Average Fixation Speed, < 150ms'),
                    self.secondary.fix_l_150_dist / self.secondary.short_fix_time)
        update_cell(file_to, new_row, key_index(df, 'Min Fixation Speed, < 150ms'), self.secondary.min_sf_speed)
        update_cell(file_to, new_row, key_index(df, 'Max Fixation Speed, < 150ms'), self.secondary.max_sf_speed)

        update_cell(file_to, new_row, key_index(df, 'Average Fixation Speed, > 150ms'),
                    self.secondary.fix_g_150_dist / (self.secondary.med_fix_time + self.secondary.long_fix_time))
        update_cell(file_to, new_row, key_index(df, 'Min Fixation Speed, > 150ms'), self.secondary.min_lf_speed)
        update_cell(file_to, new_row, key_index(df, 'Max Fixation Speed, > 150ms'), self.secondary.max_lf_speed)

        update_cell(file_to, new_row, key_index(df, 'Average Saccade Speed'),
                    self.secondary.sacc_distance / self.secondary.sacc_time)
        update_cell(file_to, new_row, key_index(df, 'Min Saccade Speed'), self.secondary.min_s_speed)
        update_cell(file_to, new_row, key_index(df, 'Max Saccade Speed'), self.secondary.max_s_speed)

        update_cell(file_to, new_row, key_index(df, 'Average Saccade Length'),
                    self.secondary.sacc_distance / len(self.saccades))
        update_cell(file_to, new_row, key_index(df, 'Min Saccade Length'), self.secondary.min_s_length)
        update_cell(file_to, new_row, key_index(df, 'Max Saccade Length'), self.secondary.max_s_length)

        update_cell(file_to, new_row, key_index(df, 'Average Saccade Time'),
                    self.secondary.sacc_time / len(self.saccades))
        update_cell(file_to, new_row, key_index(df, 'Min Saccade Time'), self.secondary.min_s_time)
        update_cell(file_to, new_row, key_index(df, 'Max Saccade Time'), self.secondary.max_s_time)

        update_cell(file_to, new_row, key_index(df, 'Average PPI'), 'none')
        update_cell(file_to, new_row, key_index(df, 'Min PPI'), 'none')
        update_cell(file_to, new_row, key_index(df, 'Max PPI'), 'none')

        print('writen')

    @staticmethod
    def ppi_to_xls(file_from, lines, file_to):
        print('excel file preparing ' + file_to)
        print('forming headline...')
        df = read_excel(file_to, engine='openpyxl')

        print('finding a row...')
        i = -1
        temp = 1
        file_from = file_from.split('/')[-1]
        for file_name in df.get('File'):
            if file_name.split('.')[0] == file_from.split('.')[0]:
                i = temp
            temp += 1
        if i == -1:
            print('no such file')
            return

        print('forming ppi')
        ppi = []
        first_fl = True
        for line in lines:
            if first_fl:
                first_fl = False
                continue
            ppi.append(float(line.split(';')[1].split('\n')[0]))

        update_cell(file_to, i, key_index(df, 'Average PPI'), sum(ppi)/len(ppi))
        update_cell(file_to, i, key_index(df, 'Min PPI'), min(ppi))
        update_cell(file_to, i, key_index(df, 'Max PPI'), max(ppi))
        print('PPI writen')

    def __str__(self):
        s = '__Metrics:__\n'
        s += 'Time Frame: ' + self.time_frame.__str__() + ' Dots count: ' + self.dots.__str__() + '\n'
        s += 'Estimated Hrz: ' + (self.dots / self.time_frame).__str__() + '\n'
        s += 'False Fix: ' + self.false_fix.__str__() + '\t'
        s += 'Ping-pong: ' + self.pong.__str__() + '\n'
        s += 'Min Speed: ' + f'{self.minSpeed:.6f}' + '\n'
        s += 'Max Speed: ' + f'{self.maxSpeed:.6f}' + '\n'
        s += 'Avr Speed: ' + f'{self.avrSpeed:.6f}' + '\n'

        fix_distance = 0
        fix_time = 0
        min_fix_speed = 1000000
        max_fix_speed = 0

        fix_min_acc = 100000
        fix_max_acc = 0
        fix_avr_acc = 0

        for fix in self.fixations:
            fix_time += fix.time
            fix_distance += fix.get_abs_distance()
            min_fix_speed = min(min_fix_speed, fix.get_min_speed())
            max_fix_speed = max(max_fix_speed, fix.get_max_speed())
            fix_min_acc = min(fix_min_acc, fix.get_min_acceleration())
            fix_max_acc = max(fix_max_acc, fix.get_max_acceleration())
            fix_avr_acc += fix.get_avr_acceleration() * fix.time

        s += 'Fix Min Speed: ' + f'{min_fix_speed:.6f}' + '\n'
        s += 'Fix Max Speed: ' + f'{max_fix_speed:.6f}' + '\n'
        s += 'Fix Avr Speed: ' + f'{fix_distance / fix_time:.6f}' + '\n'
        s += 'Fix Min Acceleration: ' + f'{fix_min_acc:.6f}' + '\n'
        s += 'Fix Max Acceleration: ' + f'{fix_max_acc:.6f}' + '\n'
        s += 'Fix Avr Acceleration: ' + f'{fix_avr_acc / fix_time:.6f}' + '\n'
        return s

    def get_section(self):
        return self.raw_data

    @staticmethod
    def get_df_null_row():
        return DataFrame({'File' : [],
                          'x_mean' : [],
                          'x_std' : [],
                          'x_min' : [],
                          'x_max' : [],
                          'x_25' : [],
                          'x_50' : [],
                          'x_75' : [],
                          'y_mean' : [],
                          'y_std' : [],
                          'y_min' : [],
                          'y_max' : [],
                          'y_25' : [],
                          'y_50' : [],
                          'y_75' : [],
                          'False Fixation, per minute': [],
                          'False Saccades, per minute': [],
                          'Saccades with amplitude > 6 degrees, per minute': [],
                          'Saccades with amplitude < 6 degrees, per minute': [],
                          'Max Curve': [],
                          'Average Curve': [],
                          'Min Curve': [], 
                          'Time Frame': [],
                          'Saccades time': [],
                          'Fixation time < 150 ms': [],
                          'Fixation time > 150 ms': [],
                          'Fixation time between 150 and 900 ms': [],
                          'Fixation time > 900 ms': [],
                          'Fixation time < 180 ms': [],
                          'Fixation time > 180 ms': [],
                          '% of Fixations < 150 ms': [],
                          '% of Fixations > 150 ms': [],
                          '% of Fixations between 150 and 900 ms': [],
                          '% of Fixations > 900 ms': [],
                          '% of Fixations < 180 ms': [],
                          '% of Fixations > 180 ms': [],
                          'Fixation time < 150 ms, per time': [],
                          'Fixation time > 150 ms, per time': [],
                          'Fixation time between 150 and 900 ms, per time': [],
                          'Fixation time > 900 ms, per time': [],
                          'Fixation time < 180 ms, per time': [],
                          'Fixation time > 180 ms, per time': [],
                          '% of Fixations < 150 ms, per minute': [],
                          '% of Fixations > 150 ms, per minute': [],
                          '% of Fixations between 150 and 900 ms, per minute': [],
                          '% of Fixations > 900 ms, per minute': [],
                          '% of Fixations < 180 ms, per minute': [],
                          '% of Fixations > 180 ms, per minute': [],
                          'Fixation Frequency': [],
                          'Average Fix Frequency in interval (1s)': [],
                          'Max Fix Frequency in interval (1s)': [],
                          'Average Acceleration': [],
                          'Min Acceleration': [],
                          'Max Acceleration': [],
                          'Average Speed': [],
                          'Min Speed': [],
                          'Max Speed': [],
                          'Average Acceleration in interval (1s)': [],
                          'Min Acceleration in interval (1s)': [],
                          'Max Acceleration in interval (1s)': [],
                          'Average Speed in interval (1s)': [],
                          'Min Speed in interval (1s)': [],
                          'Max Speed in interval (1s)': [],
                          'Average Fixation Speed': [],
                          'Min Fixation Speed': [],
                          'Max Fixation Speed': [],
                          'Average Fixation Speed, < 150ms': [],
                          'Min Fixation Speed, < 150ms': [],
                          'Max Fixation Speed, < 150ms': [],
                          'Average Fixation Speed, > 150ms': [],
                          'Min Fixation Speed, > 150ms': [],
                          'Max Fixation Speed, > 150ms': [],
                          'Average Saccade Speed': [],
                          'Min Saccade Speed': [],
                          'Max Saccade Speed': [],
                          'Average Saccade Length': [],
                          'Min Saccade Length': [],
                          'Max Saccade Length': [],
                          'Average Saccade Time': [],
                          'Min Saccade Time': [],
                          'Max Saccade Time': []})




def update_spreadsheet(path: str, _df, _new_row, sheet_name: str = "Sheet1"):
    '''

    :param path: Путь до файла Excel
    :param _df: Датафрейм Pandas для записи
    :param starcol: Стартовая колонка в таблице листа Excel, куда буду писать данные
    :param startrow: Стартовая строка в таблице листа Excel, куда буду писать данные
    :param sheet_name: Имя листа в таблице Excel, куда буду писать данные
    :return:
    '''
    wb = load_workbook(path)
    starcol = 1
    startrow = len(_df) + 1
# print(len(wb[sheet_name]))
    for ir in range(0, len(_new_row)):
        for ic in range(0, len(_new_row.iloc[ir])):
            wb[sheet_name].cell(startrow + ir, starcol + ic).value = _new_row.iloc[ir][ic]
    wb.save(path)


def update_cell(path: str, row, col, value, sheet_name: str = "Sheet1"):
    if col == -1:
        print('no key')
        return
    wb = load_workbook(path)
    wb[sheet_name].cell(row + 1, col + 1).value = value
    wb.save(path)


def key_index(data_frame, key_name):
    i = 0
    for key in data_frame.keys():
        if key == key_name:
            return i
        i += 1
    print("key wasn't found: " + key_name)
    return -1




'''
            
        all_fix = self.secondary.short_fix_count + self.secondary.med_fix_count + self.secondary.long_fix_count
        row = [ file_name, self.get_method(),
                (self.false_fix / self.time_frame * 60).__str__(),
                (self.pong / self.time_frame * 60).__str__(),
                (self.secondary.long_sacc / self.time_frame * 60).__str__(),
                (self.secondary.short_sacc / self.time_frame * 60).__str__(),
                self.secondary.max_curve.__str__(),
                (self.secondary.avr_curve / (len(self.fixations) + len(self.saccades))).__str__(),
                self.secondary.min_curve.__str__(),
                self.time_frame.__str__(),
                self.secondary.sacc_time.__str__(),
                self.secondary.short_fix_time.__str__(),
                (self.secondary.med_fix_time + self.secondary.long_fix_time).__str__(),
                self.secondary.med_fix_time.__str__(),
                self.secondary.long_fix_time.__str__(),
                self.secondary.fix_l_180_time.__str__(),
                self.secondary.fix_g_180_time.__str__(),
                (self.secondary.short_fix_count / all_fix * 100).__str__(),
                ((1 - self.secondary.short_fix_count / all_fix) * 100).__str__(),
                (self.secondary.med_fix_count / all_fix * 100).__str__(),
                (self.secondary.long_fix_count / all_fix * 100).__str__(),
                (self.secondary.fix_l_180 / all_fix * 100).__str__(),
                (self.secondary.fix_g_180 / all_fix * 100).__str__(),
                (self.secondary.short_fix_time / self.time_frame).__str__(),
                ((self.secondary.med_fix_time + self.secondary.long_fix_time) / self.time_frame).__str__(),
                (self.secondary.med_fix_time / self.time_frame).__str__(),
                (self.secondary.long_fix_time / self.time_frame).__str__(),
                (self.secondary.fix_l_180_time / self.time_frame).__str__(),
                (self.secondary.fix_g_180_time / self.time_frame).__str__(),
                (self.secondary.short_fix_count / self.time_frame * 60).__str__(),
                ((self.secondary.med_fix_count + self.secondary.long_fix_count) / self.time_frame * 60).__str__(),
                (self.secondary.med_fix_count / self.time_frame * 60).__str__(),
                (self.secondary.long_fix_count / self.time_frame * 60).__str__(),
                (self.secondary.fix_l_180 / self.time_frame * 60).__str__(),
                (self.secondary.fix_g_180 / self.time_frame * 60).__str__(),
                (len(self.fixations) / self.time_frame).__str__(),
                self.secondary.avr_freq.__str__(),
                self.secondary.max_freq.__str__(),
                self.secondary.avr_acc.__str__(),
                self.secondary.min_acc.__str__(),
                self.secondary.max_acc.__str__(),
                self.avrSpeed.__str__(),
                self.minSpeed.__str__(),
                self.maxSpeed.__str__(),
                self.secondary.avr_i_acc.__str__(),
                self.secondary.min_i_acc.__str__(),
                self.secondary.max_i_acc.__str__(),
                self.secondary.avr_i_speed.__str__(),
                self.secondary.min_i_speed.__str__(),
                self.secondary.max_i_speed.__str__(),
                (self.secondary.fix_distance / self.secondary.fix_time).__str__(),
                self.secondary.min_f_speed.__str__(),
                self.secondary.max_f_speed.__str__(),
                (self.secondary.sacc_distance / self.secondary.sacc_time).__str__(),
                self.secondary.min_s_speed.__str__(),
                self.secondary.max_s_speed.__str__(),
                (self.secondary.sacc_distance / len(self.saccades)).__str__(),
                self.secondary.min_s_length.__str__(),
                self.secondary.max_s_length.__str__(),
                (self.secondary.sacc_time / len(self.saccades)).__str__(),
                self.secondary.min_s_time.__str__(),
                self.secondary.max_s_time.__str__(),
                'none', 'none', 'none']
'''


import datetime
import pandas as pd
from sklearn.ensemble import RandomForestClassifier
from sklearn.metrics import (
    accuracy_score,
    f1_score
)

from ..Evaluate.Evaluator import Evaluator

class RandomForestEval(Evaluator):
    def __init__(self):
        self.model = None
        self.acc = -1
        self.f1 = -1
    
    def get_name(self):
        return 'RFC'
    
    def evaluate(self, data):
        if self.model == None:
            print('Not educated yet')
        else:
            return self.model.predict(data)

    def redu(self, train_X, train_Y, test_X, test_Y):
        self.model.fit(train_X, train_Y)
        y_pred = self.model.predict(test_X)
        self.acc = accuracy_score(test_Y, y_pred)
        self.f1 = f1_score(test_Y, y_pred)   

    def edu(self, train_X, train_Y, test_X, test_Y):
        err = 0
        rand = -1
        c = ['gini', 'entropy', 'log_loss']
        n = [10, 20, 30, 40, 50, 60, 70, 80, 90, 100]
        print('Start', datetime.datetime.now())
        for i in range(5):
            print(i)
            for N in n:
                for C in c:
                    model = RandomForestClassifier(n_estimators = N , criterion = C, random_state = i)
                    model.fit(train_X, train_Y)
                    y_pred = model.predict(test_X)
                    f = f1_score(test_Y, y_pred)
                    if f > err:
                        print(f)
                        rand = i
                        err = f
                        n_e = N
                        cr = C
        print('End', datetime.datetime.now())
        print(rand)
        print(n_e)
        print (cr)
        self.model = RandomForestClassifier(n_estimators = n_e , criterion = cr, random_state = rand)
        self.model.fit(train_X, train_Y)
        y_pred = model.predict(test_X)
        f = f1_score(test_Y, y_pred)
        print(f)

    def cross_edu(self, data_X, data_Y):
        teX = []
        teY = []
        trX = []
        trY = []
        step = len(data_X) // 5
        now1 = datetime.datetime.now()
        print(now1)
        for cross in range(5):                                    
            test_X = data_X.iloc[cross * step : (cross + 1) * step]
            test_Y = data_Y.iloc[cross * step : (cross + 1) * step]
            train_X = pd.concat([data_X.iloc[:cross * step], data_X.iloc[(cross + 1) * step:]], ignore_index=True)
            train_Y = pd.concat([data_Y.iloc[:cross * step], data_Y.iloc[(cross + 1) * step:]], ignore_index=True)
            teX.append(test_X)
            teY.append(test_Y)
            trX.append(train_X)
            trY.append(train_Y)
            print(len(trX[cross]))
            print(sum(trY[cross]))
            print(len(teX[cross]))
            print(sum(teY[cross]))
        # Обучение / Training
        rand = -1
        c = ['gini', 'entropy', 'log_loss']
        n = [10, 20, 30, 40, 50, 60, 70, 80, 90, 100]
        err = 0
        for i in range(1):  # перебор случайных состояний / iterating through random states
            print(i)
            for N in n:  # Перебор параметров / Iterating through the parameters
                for C in c:  # Перебор параметров / Iterating through the parameters
                    model = RandomForestClassifier(n_estimators = N , criterion = C, random_state = i)
                    f = 0
                    acc = 0
                    cur_f = 0
                    for cross in range(5): # Кроссвалидированное обучение / Cross-validated training
                        test_X = teX[cross]
                        test_Y = teY[cross]
                        train_X = trX[cross]
                        train_Y = trY[cross]
                        model.fit(train_X, train_Y)
                        y_pred = model.predict(test_X)
                        _f = f1_score(test_Y, y_pred)
                        if _f < 0.60 or _f < (cur_f-0.15): # Досрочное отбрасываение - опционально, для оптимизации
                            break
                        f += _f
                        acc += accuracy_score(test_Y, y_pred)
                    f = f / 5
                    acc = acc / 5
                    if f > err: # Сравнение текущей модели с лучшей / Comparing the current model with the best one
                        # Cохранение параметров и показателей / Saving parameters and indicators
                        print(f)
                        rand = i
                        err = f
                        n_e = N
                        cr = C
                        cross_acc = acc
        print(rand)
        print(cr)
        print(n_e)

        now = datetime.datetime.now()
        print(now)
        # Время окончания обучения / Training end timestamp 
        print('Total time:', now - now1)
        test_X = teX[cross]
        test_Y = teY[cross]
        train_X = trX[cross]
        train_Y = trY[cross]
        # обучение модели по параметрам лучший / training the model according to the best parameters
        self.model = RandomForestClassifier(n_estimators = n_e, criterion = cr, random_state = rand)
        self.model.fit(train_X, train_Y)
        y_pred = self.model.predict(test_X)
        self.f1 = f1_score(test_Y, y_pred) # результаты по одному из разбиений / results for one sample
        self.acc = accuracy_score(test_Y, y_pred)  
        self.cross_f1 = err # усреднённые результаты по всем разбиениям / averaged results for all samples
        self.cross_acc = cross_acc

    def edu_args(self, train_X, train_Y, test_X, test_Y, rand, n_e, cr):
        self.model = RandomForestClassifier(n_estimators = n_e , criterion = cr, random_state = rand)
        self.model.fit(train_X, train_Y)
        y_pred = self.model.predict(test_X)
        f = f1_score(test_Y, y_pred)
        print(f)
        
# 18
# 30
# gini


import pandas as pd
import datetime
from sklearn.tree import DecisionTreeClassifier
from sklearn.metrics import (
    accuracy_score,
    f1_score
)

from ..Evaluate.Evaluator import Evaluator

class DecisionTreeEval(Evaluator):
    def __init__(self):
        self.model = None
        self.acc = -1
        self.f1 = -1

    def get_name(self):
        return 'DTC'    
    
    def evaluate(self, data):
        if self.model == None:
            print('Not educated yet')
        else:
            return self.model.predict(data)

    def redu(self, train_X, train_Y, test_X, test_Y):
        self.model.fit(train_X, train_Y)
        y_pred = self.model.predict(test_X)
        self.acc = accuracy_score(test_Y, y_pred)
        self.f1 = f1_score(test_Y, y_pred)   

    def edu(self, train_X, train_Y, test_X, test_Y):
        err = 0
        rand = -1
        c = ['gini', 'entropy', 'log_loss']
        
        now1 = datetime.datetime.now()
        print(now1)
        for i in range(5):
            for C in c:
                model = DecisionTreeClassifier(criterion = C, random_state = i)
                model.fit(train_X, train_Y)
                y_pred = model.predict(test_X)
                m = f1_score(test_Y, y_pred)
                if m > err:
                    rand = i
                    err = m
                    cr = C
        
        now1 = datetime.datetime.now()
        print(now1)
        print(rand)
        print (cr)
        self.model = DecisionTreeClassifier(criterion = cr, random_state = rand)
        self.model.fit(train_X, train_Y)
        y_pred = model.predict(test_X)
        m = f1_score(test_Y, y_pred)
        self.acc = accuracy_score(test_Y, y_pred)
        self.f1 = f1_score(test_Y, y_pred)        
        print(m)

    def redu(self, train_X, train_Y, test_X, test_Y):
        self.model.fit(train_X, train_Y)
        y_pred = self.model.predict(test_X)
        self.acc = accuracy_score(test_Y, y_pred)
        self.f1 = f1_score(test_Y, y_pred)        


    def cross_edu(self, data_X, data_Y):
        teX = []
        teY = []
        trX = []
        trY = []
        step = len(data_X) // 5
        now1 = datetime.datetime.now()
        print(now1)
        for cross in range(5):                                    
            test_X = data_X.iloc[cross * step : (cross + 1) * step]
            test_Y = data_Y.iloc[cross * step : (cross + 1) * step]
            train_X = pd.concat([data_X.iloc[:cross * step], data_X.iloc[(cross + 1) * step:]], ignore_index=True)
            train_Y = pd.concat([data_Y.iloc[:cross * step], data_Y.iloc[(cross + 1) * step:]], ignore_index=True)
            teX.append(test_X)
            teY.append(test_Y)
            trX.append(train_X)
            trY.append(train_Y)
            print(len(trX[cross]))
            print(sum(trY[cross]))
            print(len(teX[cross]))
            print(sum(teY[cross]))
        # Обучение / Training
        err = 0
        rand = -1
        c = ['gini', 'entropy', 'log_loss']
        now1 = datetime.datetime.now()
        print(now1)
        for i in range(100): # перебор случайных состояний / iterating through random states
            print(i)
            for C in c: # Перебор параметров / Iterating through the parameters
                model = DecisionTreeClassifier(criterion = C, random_state = i)
                f = 0
                acc = 0
                cur_f = 0
                for cross in range(5): # Кроссвалидированное обучение / Cross-validated training
                    test_X = teX[cross]
                    test_Y = teY[cross]
                    train_X = trX[cross]
                    train_Y = trY[cross]
                    model.fit(train_X, train_Y)
                    y_pred = model.predict(test_X)
                    _f = f1_score(test_Y, y_pred)
                    #if _f < 0.60 or _f < (cur_f-0.15): # Досрочное отбрасываение - опционально, для оптимизации
                     #   break
                    if _f > cur_f:
                        cr = cross
                    f += _f
                    acc += accuracy_score(test_Y, y_pred)
                f = f / 5
                acc = acc / 5
                if f > err:# Сравнение текущей модели с лучшей / Comparing the current model with the best one
                    # Cохранение параметров и показателей / Saving parameters and indicators
                    rand = i
                    err = f
                    crit = C
                    cross_acc = acc
        print(rand)
        print(crit)
        
        now = datetime.datetime.now()
        print(now)
        # Время окончания обучения / Training end timestamp 
        print('Total time:', now - now1)
        test_X = teX[cross]
        test_Y = teY[cross]
        train_X = trX[cross]
        train_Y = trY[cross]
        # обучение модели по параметрам лучший / training the model according to the best parameters
        self.model = DecisionTreeClassifier(criterion = crit, random_state = i)
        self.model.fit(train_X, train_Y)
        y_pred = self.model.predict(test_X)
        self.f1 = f1_score(test_Y, y_pred) # результаты по одному из разбиений / results for one sample
        self.acc = accuracy_score(test_Y, y_pred)
        self.cross_f1 = err # усреднённые результаты по всем разбиениям / averaged results for all samples
        self.cross_acc = cross_acc

    def edu_args(self, train_X, train_Y, test_X, test_Y, rand, n_e, cr):
        self.model = DecisionTreeClassifier(n_estimators = n_e , criterion = cr, random_state = rand)
        self.model.fit(train_X, train_Y)
        y_pred = self.model.predict(test_X)
        m = accuracy_score(test_Y, y_pred)
        print(m)
     
# 18
# 30
# gini


import math

class GeoMetrics:

    def __init__(self):
        #x_mean,x_std,x_min,x_max,x_25,x_50,x_75,y_mean,y_std,y_min,y_max,y_25,y_50,y_75
        self.x_mean = self.x_std = self.x_min = self.y_mean = self.y_std = self.y_min = 0
        self.x_max = self.x_25 = self.x_50 = self.y_max = self.y_25 = self.y_50 = 0


'''
# -----------------#
x = np.array(df['norm_pos_x'])
x_mean = np.mean(x).reshape(1, 1)
x_std = np.std(x).reshape(1, 1)
x_min = np.min(x).reshape(1, 1)
x_max = np.max(x).reshape(1, 1)
x_25 = np.percentile(x, 25).reshape(1, 1)
x_50 = np.percentile(x, 50).reshape(1, 1)
x_75 = np.percentile(x, 75).reshape(1, 1)
# -----------------#
y = np.array(df['norm_pos_y'])
y_mean = np.mean(y).reshape(1, 1)
y_std = np.std(y).reshape(1, 1)
y_min = np.min(y).reshape(1, 1)
y_max = np.max(y).reshape(1, 1)
y_25 = np.percentile(y, 25).reshape(1, 1)
y_50 = np.percentile(y, 50).reshape(1, 1)
y_75 = np.percentile(y, 75).reshape(1, 1)
'''


'''
 модуль финальной оценкb данных, получаемых после работы модуля analise
 основные используемые классы: абстрактный класс Evaluator и его наследники (RandomForestEval, MLPEval, ...).
 основная функция - evaluate, 
 на вход получает данные (в формате DataFrame) со значениями рассчитанных количественных характеристик,
 где каждая строка соответствует отдельной записи или отрезку записи оператора.
 на выход - результат оценки (классификации) каждой строки 
'''



def eng_to_rus(name):
    if name == 'File':  return 'Файл'

    if name == 'x_mean':  return 'Усредненная позиция взгляда по оси X'
    if name == 'x_std':  return 'Среднее квадратичное отклонение позиции взгляда по оси X'
    if name == 'x_min':  return 'Минимальная позиция взгляда по оси X'
    if name == 'x_max':  return 'Максимальная позиция взгляда по оси X'
    if name == 'x_25':  return 'Процентиль 25 позиции взгляда по оси X'
    if name == 'x_50':  return 'Процентиль 50 позиции взгляда по оси X'
    if name == 'x_75':  return 'Процентиль 75 позиции взгляда по оси X'
    if name == 'y_mean':  return 'Усредненная позиция взгляда по оси Y'
    if name == 'y_std':  return 'Среднее квадратичное отклонение позиции взгляда по оси Y'
    if name == 'y_min':  return 'Минимальная позиция взгляда по оси Y'
    if name == 'y_max':  return 'Максимальная позиция взгляда по оси Y'
    if name == 'y_25':  return 'Процентиль 25 позиции взгляда по оси Y'
    if name == 'y_50':  return 'Процентиль 50 позиции взгляда по оси Y'
    if name == 'y_75':  return 'Процентиль 75 позиции взгляда по оси Y'
                            
    if name == 'False Fixation, per minute':  return 'Обособленные фиксации, количество в минуту'
    if name == 'False Saccades, per minute':  return 'Сверхкороткие саккады, количество в минуту'
    if name == 'Saccades with amplitude > 6 degrees, per minute':  return 'Саккады с амплитудой больше 6 градусов, количество в минуту'
    if name == 'Saccades with amplitude < 6 degrees, per minute':  return 'Саккады с амплитудой меньше 6 градусов, количество в минуту'
    if name == 'Max Curve':  return 'Максимальная кривизна траектории саккады'
    if name == 'Average Curve':  return 'Средняя кривизна траектории саккады'
    if name == 'Min Curve':  return 'Минимальная кривизна траектории саккады'
    if name == 'Time Frame':  return 'Время записи'
    if name == 'Saccades time':  return 'Суммарная длительность саккад'
    if name == 'Fixation time < 150 ms':  return 'Суммарная длительность фиксаций короче 150мс'
    if name == 'Fixation time > 150 ms':  return 'Суммарная длительность фиксаций длиннее 150мс'
    if name == 'Fixation time between 150 and 900 ms':  return 'Суммарная длительность фиксаций длиннее 150мс но короче 900мс'
    if name == 'Fixation time > 900 ms':  return 'Суммарная длительность фиксаций длиннее 900мс'
    if name == 'Fixation time < 180 ms':  return 'Суммарная длительность фиксаций короче 180мс'
    if name == 'Fixation time > 180 ms':  return 'Суммарная длительность фиксаций длиннее 180мс'
    if name == '% of Fixations < 150 ms':  return 'Доля фиксаций короче 150мс от всех фиксаций, в %'
    if name == '% of Fixations > 150 ms':  return 'Доля фиксаций длиннее 150мс от всех фиксаций, в %'
    if name == '% of Fixations between 150 and 900 ms':  return 'Доля фиксаций длиннее 150мс но короче 900мс от всех фиксаций, в %'
    if name == '% of Fixations > 900 ms':  return 'Доля фиксаций длиннее 900мс от всех фиксаций, в %'
    if name == '% of Fixations < 180 ms':  return 'Доля фиксаций короче 180мс от всех фиксаций, в %'
    if name == '% of Fixations > 180 ms':  return 'Доля фиксаций длиннее 180мс от всех фиксаций, в %'
    if name == 'Fixation time < 150 ms, per time':  return 'Доля времени (от общего времени записи) фиксаций короче 150мс'
    if name == 'Fixation time > 150 ms, per time':  return 'Доля времени (от общего времени записи) 150мс'
    if name == 'Fixation time between 150 and 900 ms, per time':  return 'Доля времени (от общего времени записи) фиксаций длиннее 150мс но короче 900мс'
    if name == 'Fixation time > 900 ms, per time':  return 'Доля времени (от общего времени записи) фиксаций длиннее 900мс'
    if name == 'Fixation time < 180 ms, per time':  return 'Доля времени (от общего времени записи) фиксаций короче 180мс'
    if name == 'Fixation time > 180 ms, per time':  return 'Доля времени (от общего времени записи) фиксаций длиннее 180мс'
    if name == "% of Fixations < 150 ms, per minute":  return 'Количество фиксаций короче 150мс на минуту записи'
    if name == '% of Fixations > 150 ms, per minute':  return 'Количество фиксаций длиннее 150мс на минуту записи'
    if name == '% of Fixations between 150 and 900 ms, per minute':  return 'Количество фиксаций длиннее 150мс но короче 900мс на минуту записи'
    if name == '% of Fixations > 900 ms, per minute':  return 'Количество фиксаций длиннее 900мс на минуту записи'
    if name == '% of Fixations < 180 ms, per minute':  return 'Количество фиксаций короче 180мс на минуту записи'
    if name == '% of Fixations > 180 ms, per minute':  return 'Количество фиксаций длиннее 180мс на минуту записи'
    if name == 'Fixation Frequency':  return 'Частота возникновения фиксаций'
    if name == 'Average Fix Frequency in interval (1s)':  return 'Средняя частота возникновения фиксаций в секундном интервале'
    if name == 'Max Fix Frequency in interval (1s)':  return 'Максимальная частота возникновения фиксаций в секундном интервале'
    if name == 'Average Acceleration':  return 'Средний модуль ускорения движения взгляда'
    if name == 'Min Acceleration':  return 'Минимальный модуль ускорения движения взгляда'
    if name == 'Max Acceleration':  return 'Максимальная модуль ускорения движения взгляда'
    if name == 'Average Speed':  return 'Средняя скорость движения взгляда'
    if name == 'Min Speed':  return 'Минимальная скорость движения взгляда'
    if name == 'Max Speed':  return 'Максимальная скорость движения взгляда'
    if name == 'Average Acceleration in interval (1s)':  return 'Средний модуль ускорения движения взгляда в секундном интервале'
    if name == 'Min Acceleration in interval (1s)':  return 'Минимальный модуль ускорения движения взгляда в секундном интервале'
    if name == 'Max Acceleration in interval (1s)':  return 'Максимальный модуль ускорения движения взгляда в секундном интервале'
    if name == 'Average Speed in interval (1s)':  return 'Средняя скорость движения взгляда в секундном интервале'
    if name == 'Min Speed in interval (1s)':  return 'Минимальная скорость движения взгляда в секундном интервале'
    if name == 'Max Speed in interval (1s)':  return 'Максимальная скорость движения взгляда в секундном интервале'
    if name == 'Average Fixation Speed':  return 'Средняя скорость движения взгляда в фиксациях'
    if name == 'Min Fixation Speed':  return 'Минимальная скорость движения взгляда в фиксациях'
    if name == 'Max Fixation Speed':  return 'Максимальная скорость движения взгляда в фиксациях'
    
    if name == 'Average Fixation Speed, < 150ms':  return 'Средняя скорость движения взгляда в фиксациях короче 150мс'
    if name == 'Min Fixation Speed, < 150ms':  return 'Минимальная скорость движения взгляда в фиксациях короче 150мс'
    if name == 'Max Fixation Speed, < 150ms':  return 'Максимальная скорость движения взгляда в фиксациях короче 150мс'                      
    if name == 'Average Fixation Speed, > 150ms':  return 'Средняя скорость движения взгляда в фиксациях длинее 150мс'
    if name == 'Min Fixation Speed, > 150ms':  return 'Минимальная скорость движения взгляда в фиксациях длинее 150мс'
    if name == 'Max Fixation Speed, > 150ms':  return 'Максимальная скорость движения взгляда в фиксациях длинее 150мс'
    if name == 'Average Saccade Speed':  return 'Средняя скорость движения взгляда в саккадах'
    if name == 'Min Saccade Speed':  return 'Минимальная скорость движения взгляда в саккадах'
    if name == 'Max Saccade Speed':  return 'Максимальная скорость движения взгляда в саккадах'
    if name == 'Average Saccade Length':  return 'Средняя длина саккады'
    if name == 'Min Saccade Length':  return 'Минимальная длина саккады'
    if name == 'Max Saccade Length':  return 'Максимальная длина саккады'
    if name == 'Average Saccade Time':  return 'Средняя длительность саккады'
    if name == 'Min Saccade Time':  return 'Минимальная длительность саккады'
    if name == 'Max Saccade Time':  return 'Максимальная длительность саккады'


                          
    print(name, 'have no rus name!')
    return name


import math

class SecondaryMetrics:

    def __init__(self):
        self.fix_distance = self.sacc_distance = 0
        self.fix_time = self.sacc_time = 0.00000001
        self.min_f_speed = self.min_s_speed = self.min_s_length = self.min_s_time = self.min_sf_speed = self.min_lf_speed = 1000000
        self.max_f_speed = self.max_s_speed = self.max_s_length = self.max_s_time = self.max_sf_speed = self.max_lf_speed = 0

        self.min_curve = self.min_acc = 100000
        self.max_curve = self.max_acc = 0
        self.avr_curve = self.avr_acc = 0
        self.sacc_length = 0
        self.long_sacc = self.short_sacc = 0
        self.short_fix_time = self.long_fix_time = self.med_fix_time = 0
        self.short_fix_count = self.long_fix_count = self.med_fix_count = 0
        self.fix_l_80 = self.fix_g_1000 = self.fix_l_180 = self.fix_g_180 = 0
        self.fix_l_180_time = self.fix_g_180_time = self.fix_l_180_count = self.fix_g_180_count = 0
        
        self.fix_l_150_dist = self.fix_g_150_dist = 0

        self.avr_freq = self.max_freq = self.min_freq = 0
        self.avr_i_speed = self.max_i_speed = 0
        self.avr_i_acc = self.max_i_acc = 0
        self.min_i_speed = self.min_i_acc = 1000000
    


import csv
import threading
import time
'''
класс для синхронизации подачи данных в режиме реального времени
'''

class Synchronizer:

    def __init__(self, path): # инициализация от пути к источнику данных
        self.path = path
        self.data = []
        self.file = open(path, newline='')
        self.reader = csv.reader(self.file, delimiter=',')
        self.prep_data()

    def prep_data(self): # подготовка данных
        i = 0
        i_time = i_x = i_y = 0
        first_line = self.reader.__next__()
        for key in first_line:  # автоматическое определение ключей данных
            if key.__contains__('time'): # ячейка времени, меняется в зависимости от формата данных окулографа
                i_time = i 
            if key.__contains__('pos_x'): # ячейка позиции по оси X, меняется в зависимости от формата данных окулографа
                i_x = i 
            if key.__contains__('pos_y'): # ячейка позиции по оси Y, меняется в зависимости от формата данных окулографа
                i_y = i
            i += 1
        row_count = 0
        for row in self.reader:
            self.data.append([row.__getitem__(i_time), row.__getitem__(i_x), row.__getitem__(i_y)])
            row_count += 1
        return self.data
    
    def start_stream(self): # запуск потока данных
        self.i = 0 # счетчик текущей позиции чтения данных
        self.time = self.data[self.i][0]
        self.last_request = 0
        self.thread = threading.Thread(target=self.run_stream)
        self.thread.start()

    def run_stream(self): # чтение данных в режиме реального времени
        while self.i < len(self.data):
            self.i += 1 # сдвиг позиции чтения данных
            self.time = self.data[self.i][0]
            time.sleep(self.time - self.data[self.i - 1][0]) # ожидание потока равно задержке между кадрами записи окулографа
    
    def request_data(self): # запрос данных выдаёт отрезок данных от предыдущего запроса до текущего
        data = self.data[self.last_request: self.i]
        print('Time since last request:',data[-1] - data[0])
        self.last_request = self.i
        





import datetime
import pandas as pd
import warnings
warnings.filterwarnings("ignore")
from sklearn.neural_network import MLPClassifier
from sklearn.metrics import (
    confusion_matrix,
    accuracy_score,
    precision_score,
    recall_score,
    f1_score
)

from ..Evaluate.Evaluator import Evaluator

class MLPEval(Evaluator):
    def __init__(self):
        self.model = None
        self.acc = -1
        self.f1 = -1

    def get_name(self):
        return 'MLP'
    
    def evaluate(self, data):
        if self.model == None:
            print('Not educated yet')
        else:
            return self.model.predict(data)

    def redu(self, train_X, train_Y, test_X, test_Y):
        self.model.fit(train_X, train_Y)
        y_pred = self.model.predict(test_X)
        self.acc = accuracy_score(test_Y, y_pred)
        self.f1 = f1_score(test_Y, y_pred)        

    def edu(self, train_X, train_Y, test_X, test_Y):
        sol = ['lbfgs', 'sgd', 'adam']
        ac = ['identity', 'logistic', 'tanh', 'relu']
        err = 0
        now1 = datetime.datetime.now()
        print(now1)
        for i in range(1):
            print(i)
            for j in range(1,20):
                for s in sol:
                    for a in ac:
                        model = MLPClassifier(solver=s, activation = a, alpha=1e-5, hidden_layer_sizes=(j), random_state=i)
                        model.fit(train_X, train_Y)
                        y_pred = model.predict(test_X)
                        f = f1_score(test_Y, y_pred)
                        if f > err:
                            print(f)
                            rand = i
                            err = f
                            h = j
                            Ac = a
                            S = s
        print(h)
        print(rand)
        print(Ac)
        print(S)
        
        now = datetime.datetime.now()
        print(now)
        print('Total time:', now - now1)

        self.model = MLPClassifier(solver=S, activation = Ac, alpha=1e-5, hidden_layer_sizes=(h), random_state=rand)
        self.model.fit(train_X, train_Y)
        y_pred = self.model.predict(test_X)
        m = accuracy_score(test_Y, y_pred)
        f = f1_score(test_Y, y_pred)
        self.acc = accuracy_score(test_Y, y_pred)
        self.f1 = f1_score(test_Y, y_pred)  
        print(m)
        print(f)

    # Обучение с кроссвалидацией / Training with crossvalidation
    def cross_edu(self, data_X, data_Y):
        now1 = datetime.datetime.now()
        print(now1) # Время начала обучения / Training start timestamp 
        # Формирования выборок для 5-фолдовой кроссвалидции
        # Formimng datasamples for 5-fold crossvalidation
        teX = []
        teY = []
        trX = []
        trY = [] 
        step = len(data_X) // 5
        for cross in range(5):                                     
            test_X = data_X.iloc[cross * step : (cross + 1) * step]
            test_Y = data_Y.iloc[cross * step : (cross + 1) * step]
            train_X = pd.concat([data_X.iloc[:cross * step], data_X.iloc[(cross + 1) * step:]], ignore_index=True)
            train_Y = pd.concat([data_Y.iloc[:cross * step], data_Y.iloc[(cross + 1) * step:]], ignore_index=True)
            teX.append(test_X)
            teY.append(test_Y)
            trX.append(train_X)
            trY.append(train_Y)
            print(len(trX[cross]))
            print(sum(trY[cross]))
            print(len(teX[cross]))
            print(sum(teY[cross]))
        # Обучение / Training
        sol = ['lbfgs', 'sgd', 'adam']
        ac = ['identity', 'logistic', 'tanh', 'relu']
        err = 0
        for i in range(1): # перебор случайных состояний / iterating through random states
            print(i)
            for j in range(1,20): # Перебор параметров / Iterating through the parameters
                for s in sol:
                    for a in ac:
                        model = MLPClassifier(solver=s, activation = a, alpha=1e-5, hidden_layer_sizes=(j), random_state=i)
                        f = 0
                        acc = 0
                        cur_f = 0
                        for cross in range(5): # Кроссвалидированное обучение / Cross-validated training
                            test_X = teX[cross]
                            test_Y = teY[cross]
                            train_X = trX[cross]
                            train_Y = trY[cross]
                            model.fit(train_X, train_Y)
                            y_pred = model.predict(test_X)
                            _f = f1_score(test_Y, y_pred)
                            if _f < 0.60 or _f < (cur_f-0.15): # Досрочное отбрасываение - опционально, для оптимизации
                                break
                            if _f > cur_f:
                                cr = cross
                            f += _f
                            acc += accuracy_score(test_Y, y_pred)
                        f = f / 5
                        acc = acc / 5
                        if f > err: # Сравнение текущей модели с лучшей / Comparing the current model with the best one
                            # Cохранение параметров и показателей / Saving parameters and indicators
                            print(f)
                            cross_acc = acc 
                            rand = i
                            err = f
                            h = j
                            Ac = a
                            S = s
        print(h)
        print(rand)
        print(Ac)
        print(S)
        
        now = datetime.datetime.now()
        print(now)
        # Время окончания обучения / Training end timestamp 
        print('Total time:', now - now1)
        test_X = teX[cr]
        test_Y = teY[cr]
        train_X = trX[cr]
        train_Y = trY[cr] 
        # обучение модели по параметрам лучший / training the model according to the best parameters
        self.model = MLPClassifier(solver=S, activation = Ac, alpha=1e-5, hidden_layer_sizes=(h), random_state=rand)
        self.model.fit(train_X, train_Y)
        y_pred = self.model.predict(test_X)
        self.f1 = f1_score(test_Y, y_pred) # результаты по одному из разбиений / results for one sample
        self.acc = accuracy_score(test_Y, y_pred)         
        self.cross_f1 = err # усреднённые результаты по всем разбиениям / averaged results for all samples
        self.cross_acc = cross_acc

    
    def edu_args(self, train_X, train_Y, test_X, test_Y, solver, activation, h_l_s, rand_state):
        #for i in range(100):
        model = MLPClassifier(solver=solver, activation=activation, alpha=1e-5, hidden_layer_sizes=(h_l_s), random_state=rand_state)
        model.fit(train_X, train_Y)
        y_pred = model.predict(test_X)
        self.acc = accuracy_score(test_Y, y_pred)
        self.f1 = f1_score(test_Y, y_pred)  
        #solver='lbfgs', activation = 'logistic', hidden_layer_sizes=(12)
        
# 18
# 30
        
# gini


import pickle
from ..Evaluate.MLPEval import MLPEval
from ..Evaluate.DecisionTreeEval import DecisionTreeEval
from ..Evaluate.RandomForestEval import RandomForestEval
from ..Evaluate.BasicEval import BasicEval
from sklearn.metrics import (
    accuracy_score,
    f1_score
)
# класс, реализующий гибридную оценку
class HybridEval():
    def __init__(self):
        self.models = []
        self.acc = -1
        self.f1 = -1


    def edu(self, train_X, train_Y, test_X, test_Y, model_names):
        if model_names.__contains__('Basic'):
            basic = BasicEval()
            basic.edu(train_X, train_Y, test_X, test_Y)
            self.models.append(basic)
            pickle.dump(basic, open('H_bsc', 'wb')) 
        if model_names.__contains__('DCT'):
            dct = DecisionTreeEval()
            dct.edu(train_X, train_Y, test_X, test_Y)
            self.models.append(dct)
            pickle.dump(dct, open('H_dct', 'wb')) 
        if model_names.__contains__('RFC'):
            rfc = RandomForestEval()
            rfc.edu(train_X, train_Y, test_X, test_Y)
            self.models.append(rfc)
            pickle.dump(rfc, open('H_rfc', 'wb')) 
        if model_names.__contains__('MLP'):
            mlp = MLPEval()
            mlp.edu(train_X, train_Y, test_X, test_Y)
            self.models.append(mlp)
            pickle.dump(mlp, open('H_mlp', 'wb')) 
        
    def load(self, model_names):
        if model_names.__contains__('Basic'):
            basic = pickle.load(open('H_bsc', 'rb')) 
            self.models.append(basic)
        if model_names.__contains__('DCT'):
            dct = pickle.load(open('H_dct', 'rb')) 
            self.models.append(dct)
        if model_names.__contains__('RFC'):
            rfc = pickle.load(open('H_rfc', 'rb')) 
            self.models.append(rfc)
        if model_names.__contains__('MLP'):
            mlp = pickle.load(open('H_mlp', 'rb')) 
            self.models.append(mlp)

    def evaluate(self, data, treshold = 0.1):
        preds = []
        for model in self.models:
            y_pred = model.evaluate(data)
            preds.append(y_pred)
        
        eval = []
        
        for j in range(len(preds[0])):
            p_sum = preds[0][j]
            for i in range(1,len(self.models)):
                p_sum += preds[i][j]
            if p_sum == 0 or p_sum == len(self.models):
                eval.append(preds[0][j])
            else:
                if p_sum/len(self.models) >= treshold:
                    eval.append(1)
                else:
                    eval.append(0)
                
        return eval
    
    def eval(self, test_X, test_Y):
        preds = []
        for model in self.models:
            y_pred = model.evaluate(test_X)
            preds.append(y_pred)
            self.f1 = f1_score(test_Y, y_pred) # результаты по одному из разбиений / results for one sample
            self.acc = accuracy_score(test_Y, y_pred)
        
        sure = []
        unsure = []
        test_sure = []
        test_unsure = []
        for j in range(len(preds[0])):
            p_sum = preds[0][j]
            for i in range(1,len(self.models)):
                p_sum += preds[i][j]
            if p_sum == 0 or p_sum == len(self.models):
                sure.append(preds[0][j])
                test_sure.append(test_Y['AU_cat'][j])
            else:
                unsure.append(p_sum / len(self.models))
                test_unsure.append(test_Y['AU_cat'][j])

        print('Sure:', format(len(sure) / (len(sure) + len(unsure)) * 100, '.2f') + '%', 'F1:', f1_score(sure, test_sure))
        unsure = self.filter(unsure, 0.1)
        print('UnSure:', format(len(unsure) / (len(sure) + len(unsure)) * 100, '.2f') + '%', 'F1:', f1_score(unsure, test_unsure))
        
    def filter(self, pred, treshold):
        for i in range(len(pred)):
            if pred[i] >= treshold:
                pred[i] = 1
            else:
                pred[i] = 0
        return pred



#D:\PythonProjects\Faeyeton\_Data


import csv
import random

# Константы:
GEN = 20 # количество поколений
Ch_Count = 10000 # количество потомков
delta = 2.0 # максимально отклонение веса одного признака в одном поколении
X_Number = 9 # количество признаков

# обычная регрессия,
class Func:
    def __init__(self, basis_weights=None, weights_delta=None):
        self.weights = []
        if basis_weights is None:
            self.weights = [2028, 2529, -1166, 335, -2335, -2411, -2322, -1576, -2391] # по корреляции

            '''
            for i in range(0, 9):  # у нас 9 признаков, заведём 9 весов
                self.weights.append(1.0)  # стартовые значения весов
            '''
        else:
            for weight in basis_weights:
                self.weights.append(weight * (random.random() - 0.5) * 2 * weights_delta)

    def calc(self, xs):
        value = 0
        for i in range(len(self.weights)):
            value += self.weights[i] * float(xs[i].replace(',', '.'))
        return value

    def __str__(self):
        return 'F:' + self.weights.__str__()
'''

def eval_func(func, work_list, _print=False):
    eval_list = []
    control_list = []
    for i in range(len(work_list)):
        eval_list.append([work_list[i][0], random.randint(1, 4)]) # func.calc(work_list[i][1][:9])
        control_list.append([work_list[i][0], float(work_list[i][1][-2].replace(',', '.'))])
    eval_list = sorted(eval_list, key=lambda x: x[-1])
    control_list = sorted(control_list, key=lambda x: x[-1])
    if _print:
        print('pre Q')
        print(eval_list)
        print(control_list)
    for i in range(len(eval_list)):
        eval_list[i].append(i * 4 // len(eval_list) + 1)
        control_list[i].append(i * 4 // len(eval_list) + 1)
    q = 0
    eval_list = sorted(eval_list, key=lambda x: x[0])
    control_list = sorted(control_list, key=lambda x: x[0])

    if _print:
        print('post Q')
        print(eval_list)
        print(control_list)

    for i in range(len(eval_list)):
        if control_list[i][-1] == eval_list[i][-1]:
            q += 1
    return q / len(eval_list)

'''
def eval_func(func, work_list):
    control_list = sorted(work_list, key=lambda x: float(x[1][9].replace(',', '.')))
    eval_list = work_list.copy()
    for i in range(len(eval_list)):
        eval_list[i].append(func.calc(eval_list[i][1][:9]))
    eval_list = sorted(eval_list, key=lambda x: x[-1])
    q = 0
    for i in range(len(eval_list)):
        if control_list[i][0] == eval_list[i][0]:
            q += 1
    return q / len(eval_list)


path = 'ann.csv'
reader = csv.reader(open(path, newline=''), delimiter=';')
#first_line = reader.__next__()
#print(first_line)
matrix = []
for row in reader:
    t_row = [row[0], row[2:]]
    matrix.append(t_row)#.__getitem__(2))

matrix = matrix[1:]
print(matrix) # входные данные
random.shuffle(matrix) # перемешаем данные

start = Func()
best = start
best_dist = eval_func(start, matrix[:len(matrix) * 8 // 10])
print(start, ' : ', best_dist)
I = 0
# for gen in range(0, GEN):      # фиксированное количество поколений
while best_dist < 0.8:           # считать, пока дистанция не сократится
    for ch_i in range(0, Ch_Count):
        child = Func(start.weights, delta)
        ch_dist = eval_func(child, matrix[:len(matrix) * 8 // 10]) # обучаем на 80% данных
        if ch_dist > best_dist:
            best_dist = ch_dist
            best = child
    if start == best:
        I += 1
        delta *= delta
        if I >= 15:
            start = Func()
            best = start
            best_dist = eval_func(start, matrix[:len(matrix) * 8 // 10])
            print('Reset')
            I = 0
            delta = 2.0
    else:
        I = 0
        start = best
        print(start, ' : ', best_dist)

print(best_dist)
print(eval_func(best, matrix[len(matrix) * 8 // 10+1:], True)) # контроль на 20% данных

for row in matrix:
    row.append(str(best.calc(row[1][:9])))

print(eval_func(best, matrix))

#print(our_list)

# F:[2028, 2529, -1166, 335, -2335, -2411, -2322, -1576, -2391]  :  0.3877551020408163
# F:[9516.496548884677, -1481.7470158300598, -24799.939347375548, -496.0585053880757, -8.202418612181155, -1827.531572733274, 2215.4052451615994, -35430.470695843134, 4732.063298066532]  :  0.6122448979591837


# модуль анализа входных данных, выделяющий интересующие нас метрики
from ..Input import read_csv_file
from ..Analise.ParsedData import ParsedData


# стадия 1

def clear(input_section):
    # первичная обработка данных
    return input_section


def interpolation(input_section):
    # первичная обработка данных
    return input_section


def parse_ff_tf(file_from, file_to):
    section = read_csv_file(file_from)
    metrics = ParsedData()
    metrics.parse(section, 3, 5)
    metrics.to_csv(file_from, file_to)


def check_interpolation():
    section = read_csv_file('../test_data1.csv')
    s = 0
    d = 0.001
    for i in range(1, len(section.positionData) - 1):
        if abs(section.positionData[i].get_distance(section.positionData[i - 1])
               - section.positionData[i].get_distance(section.positionData[i + 1])) < d:
            s += 1
    print("Suspicion: " + (s / len(section.positionData)).__str__())
    print(s)

'''
section = read_csv_file('D:/PythonProjects/Participant 3/22-12-26-game-afternoon.csv')
metrics = ParsedData()
metrics.parse(section, 2, 0.1)
metrics.calc_metrics()
metrics.to_xls_by_row('test.csv', '../UI/masterfile.xlsx')
'''

'''
section = read_csv_file('../test_data1.csv')
metrics = ParsedData()
metrics.parse(section, 1, 5, 2.5)
metrics.to_xls('test_data1', '../UI/masterfile.xlsx')
'''


import math

class EyeMovement:
    FOV = 82 * math.sqrt(2)

    def __init__(self, movement_type):
        self.movementType = movement_type
        self.points = []
        self.distances = []
        self.time = 0
        self.velocities = []
        self.accelerations = []

    def add_point(self, point):
        self.points.append(point)
        if len(self.points) < 2:
            return
        self.time = point.time - self.points[0].time
        self.distances.append(self.points[-1].get_distance(self.points[-2]))
        self.velocities.append(self.distances[-1] / (self.points[-1].time - self.points[-2].time))
        if len(self.velocities) < 2:
            return
        self.accelerations.append(
            abs(self.velocities[-1] - self.velocities[-2]) / (self.points[-1].time - self.points[-2].time))

    def get_avr_speed(self):
        return sum(self.distances) / self.time

    def get_med_speed(self):
        return sum(self.velocities) / len(self.velocities)

    def get_min_speed(self):
        return min(self.velocities)

    def get_max_speed(self):
        return max(self.velocities)

    def get_avr_acceleration(self):
        acc = 0
        for i in range(2, len(self.points)):
            acc += self.accelerations[i-2] * (self.points[i].time - self.points[i-1].time)
        return acc / self.time

    def get_min_acceleration(self):
        if len(self.accelerations) > 0:
            return min(self.accelerations)
        else:
            return 1000000

    def get_max_acceleration(self):
        if len(self.accelerations) > 0:
            return max(self.accelerations)
        else:
            return 0

    def get_abs_distance(self):
        if len(self.points) < 2:
            print(' error ')
            return 1
        return self.points[-1].get_distance(self.points[0])

    def get_distance(self):
        return sum(self.distances)

    def in_center(self, center_radius):
        x = y = 0
        for p in self.points:
            x += p.x
            y += p.y
        x /= len(self.points)
        y /= len(self.points)
        return math.sqrt(pow((self.FOV / 2) - x, 2) + pow((self.FOV / 2) - y, 2)) < center_radius

    def get_curve(self):
        if self.get_abs_distance() == 0:
            print('error')
            return 1
        return sum(self.distances) / self.get_abs_distance()

    def get_max_dist(self):
        max_x = min_x = self.points[0].x
        max_y = min_y = self.points[0].y
        for point in self.points:
            max_x = max(max_x, point.x)
            min_x = min(min_x, point.x)
            max_y = max(max_y, point.y)
            min_y = min(min_y, point.y)
        return max(max_y - min_y, max_x - min_x)

    def get_area_dist(self):
        start = self.points[0]
        end = self.points[-1]
        return pow(pow(end.x - start.x, 2) + pow(end.y - start.y, 2), 0.5)


class HeadMovement(EyeMovement):
    def __init__(self, movement_type):
        super.__init__(movement_type)
        self.angles = []
        self.angle_distances = []
        self.angle_velocities = []
        self.angle_accelerations = []

    def add_point_angle(self, angle):
        
        self.angles.append(angle)
        if len(self.angles) < 2:
            return
        self.angle_distances.append(abs(self.angles[-1] - self.angles[-2]))
        self.angle_velocities.append(self.angle_distances[-1] / (self.points[-1].time - self.points[-2].time))
        if len(self.angle_velocities) < 2:
            return
        self.accelerations.append(
            abs(self.angle_velocities[-1] - self.angle_velocities[-2]) / (self.points[-1].time - self.points[-2].time))

    def get_avr_angle_speed(self):
        return sum(self.angle_distances) / self.time

    def get_med_angle_speed(self):
        return sum(self.angle_velocities) / len(self.angle_velocities)

    def get_min_angle_speed(self):
        return min(self.angle_velocities)

    def get_max_angle_speed(self):
        return max(self.angle_velocities)

    def get_avr_angle_acceleration(self):
        acc = 0
        for i in range(2, len(self.angles)):
            acc += self.angle_accelerations[i-2] * (self.points[i].time - self.points[i-1].time)
        return acc / self.time

    def get_min_angle_acceleration(self):
        if len(self.angle_accelerations) > 0:
            return min(self.angle_accelerations)
        else:
            return 1000000

    def get_max_angle_acceleration(self):
        if len(self.angle_accelerations) > 0:
            return max(self.angle_accelerations)
        else:
            return 0

    def get_abs_angle_distance(self):
        if len(self.points) < 2:
            print(' error ')
            return 1
        return abs(self.angles[-1] - self.angles[0])

    def get_angle_distance(self):
        return sum(self.angle_distances)


from Analise import read_csv_file, ParsedData
import csv


def test_1():
    print('Test1 \n Basic input, Parsing by dist, arg=0.1, Standard output to .csv')
    section = read_csv_file('TestSamples/sample_data1.csv')
    metrics = ParsedData()
    metrics.parse(section, 2, 0.1)
    metrics.calc_metrics()
    metrics.to_csv('sample_data1.csv', 'TestSamples/sample_output.csv')


def test_2():
    print('Test2 \n Basic input, Parsing by area, arg=1, Standard output to .xlsx by row')
    section = read_csv_file('TestSamples/sample_data1.csv')
    metrics = ParsedData()
    metrics.parse(section, 1, 1)
    metrics.calc_metrics()
    metrics.to_xls_by_row('sample_data1.csv', 'TestSamples/sample_output.xlsx')


def test_3():
    print('Test3 \n Basic input, Parsing by speed, arg=30, Standard output to .xlsx by updating')
    section = read_csv_file('TestSamples/sample_data1.csv')
    metrics = ParsedData()
    metrics.parse(section, 0, 30, 2.5)
    metrics.to_xls('test_data1', 'TestSamples/masterfile.xlsx')


def test_4():
    print('Test4 \n Wrong input, Parsing by dist, 0.1, Standard output to .csv')
    section = read_csv_file('TestSamples/nullname')
    metrics = ParsedData()
    metrics.parse(section, 2, 0.1)
    metrics.to_csv('sample_data1.csv', 'TestSamples/sample_output.csv')


def test_5():
    print('Test5 \n Basic input, Parsing by dist, 0.1, Wrong output to .xlsx')
    section = read_csv_file('TestSamples/sample_data1.csv')
    metrics = ParsedData()
    metrics.parse(section, 2, 0.1)
    metrics.to_csv('sample_data1.csv', 'TestSamples/nullname.xlsx')


def test_6():
    print('Test6 \n Basic input, Parsing by dist, arg=0.1, Standard output to .csv')

    print('Comparing results')
    section = read_csv_file('TestSamples/sample_data1.csv')
    metrics = ParsedData()
    metrics.parse(section, 2, 0.1)
    metrics.calc_metrics()
    metrics.to_csv('sample_data1.csv', 'TestSamples/sample_output.csv')
    # compare with

    reader_last = csv.reader(open('TestSamples/sample_output.csv', newline=''), delimiter=',')
    data_last = []
    reader_standard = csv.reader(open('TestSamples/standard.csv', newline=''), delimiter=',')
    data_standard = []
    for row in reader_last:
        data_last = row
    for row in reader_standard:
        data_standard = row

    '''
    print(data_last)
    print(data_standard)
    '''

    fl = len(data_last) == len(data_standard)
    for i in range(0, len(data_last)):
        fl = fl and (data_last[i] == data_standard[i])
    print(fl)


test_6()



import sys
from time import sleep
import pandas as pd
import pickle
from ..Analise.Names import eng_to_rus
from ..Input import read_csv_file
from ..Analise.ParsedData import ParsedData
from ..Evaluate.RandomForestEval import RandomForestEval
from ..Evaluate.DecisionTreeEval import DecisionTreeEval
from ..Evaluate.MLPEval import MLPEval
from ..Evaluate.BasicEval import BasicEval
from PyQt6.QtGui import QPixmap
from PyQt6.QtWidgets import *
from sklearn.metrics import (
    confusion_matrix,
    accuracy_score,
    precision_score,
    recall_score,
    f1_score
)

class EvalApp(QMainWindow):
    def __init__(self):
        super().__init__()

        self.rb_base = QRadioButton('Базовый', self)
        self.rb_rfc = QRadioButton('Random Forest', self)
        self.rb_dtc = QRadioButton('Decision Tree', self)
        self.rb_mlp = QRadioButton('MLP', self)

        self.rb_group = QButtonGroup()
        self.rb_group.addButton(self.rb_base)
        self.rb_group.addButton(self.rb_rfc)
        self.rb_group.addButton(self.rb_dtc)
        self.rb_group.addButton(self.rb_mlp)
        self.rb_mlp.setChecked(True)
        self.model_name = 'Basic'
        self.rb_group.buttonClicked.connect(self.rb_click)

        self.data_list1 = []
        self.data_list2 = []
        self.data_list_text1 = QPlainTextEdit(self)
        self.data_list_text2 = QPlainTextEdit(self)
        self.model = None
        self.edu_list1 = None
        self.edu_list2 = None
        self.edu_list_text1 = QPlainTextEdit(self)
        self.edu_list_text2 = QPlainTextEdit(self)

        self.eval_list = None
        self.eval_list_text = QPlainTextEdit(self)

        self.res_list1 = ParsedData.get_df_null_row()
        self.res_list2 = ParsedData.get_df_null_row()
        self.res_list_text1 = QPlainTextEdit(self)
        self.res_list_text2 = QPlainTextEdit(self)
        self.arg_line = QLineEdit('1.0', self)
        self.file_size_line = QLineEdit('нет', self)

        self.corr = 0
        self.err = 0

        self.keys = ParsedData.get_df_null_row().drop(columns=['File'])
        self.keys.loc[0] = [0] * len(self.keys.columns)
        self.key_boxes = []
        
        self.ideal_min = ParsedData.get_df_null_row().drop(columns=['File'])
        self.ideal_max = ParsedData.get_df_null_row().drop(columns=['File'])
        self.f1_line = QLineEdit('0.0', self)

        self.data_group = []
        self.edu_group = []
        self.eval_group = []
        self.res_group = []
        self.status_ = 0
        self.status_img = QLabel(self)
        self.status_text = QLabel(self)
        self.reset_status_btn = QPushButton('Сбросить', self)
        self.reset_status_btn.clicked.connect(lambda : self.set_status(0))

        self.setup_ui()

    def setup_ui(self):
        self.setGeometry(300, 300, 1010, 650)
        self.setWindowTitle('Оценка данных')

        data_label1 = QLabel('Бодрые', self)
        data_label1.setGeometry(80, 10, 200, 15)
        self.data_group.append(data_label1)

        data_files_btn = QPushButton('Загрузить данные бодрых', self)
        data_files_btn.setGeometry(10, 25, 200, 30)
        data_files_btn.clicked.connect(self.calc_data1)
        self.data_group.append(data_files_btn)

        self.data_list_text1.setStyleSheet("color: blue; background-color: white;")
        self.data_list_text1.setGeometry(10, 60, 200, 150)
        self.data_group.append(self.data_list_text1)

        data_label2 = QLabel('Усталые', self)
        data_label2.setGeometry(80, 210, 90, 15)
        self.data_group.append(data_label2)

        data_files_btn = QPushButton('Загрузить данные усталых', self)
        data_files_btn.setGeometry(10, 225, 200, 30)
        data_files_btn.clicked.connect(self.calc_data2)
        self.data_group.append(data_files_btn)

        self.data_list_text2.setStyleSheet("color: red; background-color: white;")
        self.data_list_text2.setGeometry(10, 260, 200, 150)
        self.data_group.append(self.data_list_text2)

        data_save_file_btn = QPushButton('Сохранить\nпосчитанные\nданные', self)
        data_save_file_btn.setGeometry(10, 420, 100, 60)
        data_save_file_btn.clicked.connect(self.save_data)
        self.data_group.append(data_save_file_btn)

        data_load_file_btn = QPushButton('Загрузить\nпосчитанные\nданные', self)
        data_load_file_btn.setGeometry(110, 420, 100, 60)
        data_load_file_btn.clicked.connect(self.load_data)
        self.data_group.append(data_load_file_btn)

        arg_label = QLabel('Размер области фиксации:', self)
        arg_label.setGeometry(10, 480, 150, 30)
        self.data_group.append(arg_label)
        self.arg_line.setGeometry(160, 480, 50, 30)
        self.data_group.append(self.arg_line)

        file_size_label = QLabel('Максимальный размер файла:', self)
        file_size_label.setGeometry(10, 520, 150, 30)
        self.data_group.append(file_size_label)
        self.file_size_line.setGeometry(160, 520, 50, 30)
        self.data_group.append(self.file_size_line)

    ##################################################################################################################

        edu_files_btn = QPushButton('Разбить выборку для обучения', self)
        edu_files_btn.setGeometry(280, 10, 200, 30)
        edu_files_btn.clicked.connect(self.take_sample)
        self.edu_group.append(edu_files_btn)

        edu_label1 = QLabel('Бодрые', self)
        edu_label1.setGeometry(360, 40, 200, 15)
        self.edu_group.append(edu_label1)
        self.edu_list_text1.setStyleSheet("color: blue; background-color: white;")
        self.edu_list_text1.setGeometry(280, 60, 200, 150)
        self.edu_group.append(self.edu_list_text1)
        edu_label2 = QLabel('Усталые', self)
        edu_label2.setGeometry(360, 200, 90, 50)
        self.edu_group.append(edu_label2)
        self.edu_list_text2.setStyleSheet("color: red; background-color: white;")
        self.edu_list_text2.setGeometry(280, 240, 200, 150)
        self.edu_group.append(self.edu_list_text2)
        
        edu_btn = QPushButton('Обучение', self)
        edu_btn.setGeometry(280, 400, 200, 30)
        edu_btn.clicked.connect(self.do_edu)
        self.edu_group.append(edu_btn)

        edu_save_file_btn = QPushButton('Сохранить\n модель', self)
        edu_save_file_btn.setGeometry(280, 430, 100, 60)
        edu_save_file_btn.clicked.connect(self.save_edu_file)
        self.edu_group.append(edu_save_file_btn)
        edu_load_file_btn = QPushButton('Загрузить\n модель', self)
        edu_load_file_btn.setGeometry(380, 430, 100, 60)
        edu_load_file_btn.clicked.connect(self.load_edu_file)
        self.edu_group.append(edu_load_file_btn)

        self.crossvalid_box = QCheckBox('Кроссвалидация', self)
        self.crossvalid_box.setGeometry(280, 500, 200, 30)
        self.edu_group.append(self.crossvalid_box)
        self.normalize_box = QCheckBox('Нормализация', self)
        self.normalize_box.setGeometry(280, 540, 200, 30)
        self.edu_group.append(self.normalize_box)
        #crossvalid_box.clicked.connect(self.model_cross_validation)
        

    ##################################################################################################################

        eval_label = QLabel('На оценку', self)
        eval_label.setGeometry(550, 40, 200, 10)
        self.eval_list_text.setGeometry(550, 50, 200, 150)
        self.eval_group.append(eval_label)

        self.rb_base.move(550, 210)
        self.rb_rfc.move(550, 240)
        self.rb_dtc.move(550, 270)
        self.rb_mlp.move(550, 300)

        choose_all_btn = QPushButton('Всё', self)
        choose_all_btn.setGeometry(550, 370, 50, 30)
        choose_all_btn.clicked.connect(self.check_box_all)
        self.eval_group.append(choose_all_btn)

        choose_std_btn = QPushButton('Стандартный набор', self)
        choose_std_btn.setGeometry(610, 370, 200, 30)
        choose_std_btn.clicked.connect(self.check_box_std)
        self.eval_group.append(choose_std_btn)

        # Заглушка
        choose_find = QLineEdit('..', self)
        choose_find.setGeometry(820, 370, 180, 30)
        self.eval_group.append(choose_find)

        scroll_box = QScrollArea(self)
        scroll_box.setGeometry(550, 410, 430, 190)
        self.eval_group.append(scroll_box)

        self.scrollbar = QScrollBar(self)
        self.scrollbar.setGeometry(980, 410, 20, 190)
        self.scrollbar.sliderMoved.connect(self.scroll_check_box)
        self.eval_group.append(self.scrollbar)

        self.create_check_box(scroll_box)        

        res_files_btn = QPushButton('Выполнить оценку', self)
        res_files_btn.setGeometry(550, 10, 200, 30)
        res_files_btn.clicked.connect(self.do_eval)        
        self.eval_group.append(res_files_btn)

        res_label = QLabel('Результаты оценки', self)
        res_label.setGeometry(840, 10, 200, 15)
        self.eval_group.append(res_label)
        res_label1 = QLabel('Бодрые', self)
        res_label1.setGeometry(880, 30, 200, 10)
        self.eval_group.append(res_label1)
        self.res_list_text1.setStyleSheet("color: blue; background-color: white;")
        self.res_list_text1.setGeometry(800, 50, 200, 60)
        self.eval_group.append(self.res_list_text1)

        res_label2 = QLabel('Усталые', self)
        res_label2.setGeometry(880, 115, 90, 15)
        self.eval_group.append(res_label2)
        self.res_list_text2.setStyleSheet("color: red; background-color: white;")
        self.res_list_text2.setGeometry(800, 130, 200, 60)
        self.eval_group.append(self.res_list_text2)

        f1_label = QLabel('Точность оценки:',self)
        f1_label.setGeometry(800, 200, 100, 30)
        self.eval_group.append(f1_label)
        self.f1_line.setGeometry(900, 200, 100, 30)
        self.eval_group.append(self.f1_line)
        f1_label = QLabel('Кроссвалидация:',self)
        f1_label.setGeometry(800, 240, 100, 30)
        self.crossvalid_label = QLineEdit('', self)
        self.crossvalid_label.setGeometry(900, 240, 100, 30)
        self.eval_group.append(self.crossvalid_label)

        self.set_status(0)
        self.check_box_std()
        self.show()

    def rb_click(self, rb):
        if rb.text() == 'Базовый':
            self.model_name = 'Basic'
        elif rb.text() == 'Random Forest':
            self.model_name = 'RFC'
        elif rb.text() == 'Decision Tree':
            self.model_name = 'DTC'
        elif rb.text() == 'MLP':
            self.model_name = 'MLP'

    def create_check_box(self, parent):
        x = 0
        y = 0
        w = parent.width()
        h = 15
        self.scr_area = QScrollArea(parent)
        for col in ParsedData.get_df_null_row().drop(columns='File').head():
            chbx = QCheckBox(eng_to_rus(str(col)), self.scr_area)
            chbx.setChecked(self.keys[col].iloc(0) == 1)
            chbx.setGeometry(x, y, w, h)
            chbx.clicked.connect(self.check_box_click)
            self.key_boxes.append(chbx)
            y += h 
        self.scr_area.resize(parent.width(), y)
        self.scrollbar.setMaximum(y - parent.height() + 15)
        self.scrollbar.setMinimum(-8)
    
    def scroll_check_box(self):
        value = self.scrollbar.value()
        self.scr_area.move(0, -value)

    def check_box_click(self):
        i = 0
        for col in self.keys.head():
            self.keys[col][0] = 1 if self.key_boxes[i].isChecked() else 0
            i += 1
    
    def check_box_all(self):
        i = 0
        fl = not self.key_boxes[i].isChecked()
        for col in self.keys.head():
            self.keys[col][0] = 1 if fl else 0
            self.key_boxes[i].setChecked(fl)
            i += 1
    
    def check_box_std(self):
        std = {'x_mean', 'x_std', 'x_min', 'x_max', 'x_25', 'x_50', 'x_75', 'y_mean', 'y_std', 'y_min', 'y_max', 'y_25', 'y_50', 'y_75', 'Saccades with amplitude < 6 degrees, per minute', 'Max Curve', 'Fixation time > 150 ms', 'Fixation time > 180 ms', '% of Fixations < 150 ms', '% of Fixations > 150 ms', '% of Fixations > 900 ms', '% of Fixations < 180 ms', '% of Fixations > 180 ms', 'Fixation time < 150 ms, per time', 'Fixation time > 150 ms, per time', '% of Fixations > 150 ms, per minute', '% of Fixations < 180 ms, per minute', 'Min Speed', 'Max Speed', 'Average Speed in interval (1s)', 'Max Speed in interval (1s)', 'Average Fixation Speed', 'Max Fixation Speed', 'Average Saccade Length', 'Min Saccade Length', 'Max Saccade Length'}
        #std = {'x_mean', 'x_std', 'x_min', 'x_max', 'x_25', 'x_50', 'x_75', 'y_mean', 'y_std', 'y_min', 'y_max', 'y_25', 'y_50', 'y_75'}
        #std = {'x_mean', 'y_mean', 'Average Curve', 'Min Curve', 'Min Saccade Time', 'Average Fixation Speed', "% of Fixations < 150 ms, per minute", 'Average Fixation Speed, < 150ms'}
        i = 0
        for col in self.keys.head():
            self.keys[col][0] = 1 if col in std else 0
            self.key_boxes[i].setChecked(col in std)
            i += 1

    def calc_df(self, files):
        arg = float(self.arg_line.text())
        data_frame = ParsedData.get_df_null_row()
        file_size = -1  if self.file_size_line.text() == 'нет' else float(self.file_size_line.text())
        for line in files:
            p = 1
            print(file_size)
            for section in read_csv_file(line).split(file_size):
                # print(section.time_frame())
                # print(section)
                metrics = ParsedData()
                metrics.parse(section, 2, arg)
                metrics.calc_metrics()
                row = metrics.get_df_row(line.split('/')[-1] + ('' if file_size == -1 else 'part '+str(p)))
                p += 1
                data_frame = pd.concat([data_frame, row], ignore_index=True)
        return data_frame

    def calc_data1(self):
        csv_path, _ = QFileDialog.getOpenFileNames(caption='Загрузите Бодрых')
        self.data_list1 = self.calc_df(csv_path)
        self.data_list_text1.setPlainText('')
        for line in self.data_list1['File']:
            self.data_list_text1.setPlainText(self.data_list_text1.toPlainText() + line + '\n')
    
    def calc_data2(self):
        csv_path, _ = QFileDialog.getOpenFileNames(caption='Загрузите Уставших')
        self.data_list2 = self.calc_df(csv_path)
        self.data_list_text2.setPlainText('')
        for line in self.data_list2['File']:
            self.data_list_text2.setPlainText(self.data_list_text2.toPlainText() + line + '\n')
        self.set_status(1)

    def save_data(self):
        file_name = QFileDialog.getSaveFileName(directory='_Data')
        print(file_name)
        # self.data_list1.to_csv(file_name[0] + '.csv', sep=';', index=False)
        d1 = self.data_list1.drop(columns=['File'])
        d2 = self.data_list2.drop(columns=['File'])
        gen = pd.concat([d1, d2],axis=0)
        d1 = (d1-gen.mean())/gen.std() 
        d2 = (d2-gen.mean())/gen.std() 
        d1 = pd.concat([self.data_list1['File'], d1], axis=1)
        d2 = pd.concat([self.data_list2['File'], d2], axis=1)
        self.data_list1.to_csv(file_name[0] + ' Бодрые.csv', sep=';', index=False)
        self.data_list2.to_csv(file_name[0] + ' Усталые.csv', sep=';', index=False)
        d1.to_csv(file_name[0] + ' Бодрые Норм.csv', sep=';', index=False)
        d2.to_csv(file_name[0] + ' Усталые Норм.csv', sep=';', index=False)

    def load_data(self):
        file_name = QFileDialog.getOpenFileName(directory='_Data', caption='Бодрые')
        self.data_list1 = pd.read_csv(file_name[0], sep=';')
        file_name = QFileDialog.getOpenFileName(directory='_Data', caption='Уставшие')
        self.data_list2 = pd.read_csv(file_name[0], sep=';')
        self.data_list_text1.setPlainText('')
        for line in self.data_list1['File']:
            self.data_list_text1.setPlainText(self.data_list_text1.toPlainText() + str(line) + '\n')
        self.data_list_text2.setPlainText('')
        for line in self.data_list2['File']:
            self.data_list_text2.setPlainText(self.data_list_text2.toPlainText() + str(line) + '\n')
        self.set_status(1)

    def take_sample(self):
        print('load edu button click')
        #self.eval_list = ParsedData.get_df_null_row() # pd.concat([self.data_list1, self.data_list2])
        self.edu_list1 = self.data_list1.sample(n=int(0.8*len(self.data_list1)))
        self.edu_list2 = self.data_list2.sample(n=int(0.8*len(self.data_list2)))
        ev1 = pd.concat([self.data_list1, self.edu_list1]).drop_duplicates(keep=False)
        ev2 = pd.concat([self.data_list2, self.edu_list2]).drop_duplicates(keep=False)
        self.eval_list = pd.concat([ev1, ev2], ignore_index=True)
        print(len(self.data_list1))
        print(len(self.data_list2))
        print(len(self.edu_list1))
        print(len(self.edu_list2))
        print(len(self.eval_list))
        self.edu_list_text1.setPlainText('')
        for _, row in self.edu_list1.iterrows():
            self.edu_list_text1.setPlainText(self.edu_list_text1.toPlainText() + str(row['File']) + '\n')            
        self.edu_list_text2.setPlainText('')
        for _, row in self.edu_list2.iterrows():
            self.edu_list_text2.setPlainText(self.edu_list_text2.toPlainText() + str(row['File']) + '\n')

        #print(self.eval_list)
        self.eval_list_text.setPlainText('')
        for _, row in self.eval_list.iterrows():
            self.eval_list_text.setPlainText(self.eval_list_text.toPlainText() + str(row['File']) + '\n')        
            
    def do_edu(self):
        self.do_model_edu()           

    def save_edu_file(self):
        pkl_filename = QFileDialog.getSaveFileName(directory='_Models')[0]
        with open(pkl_filename, 'wb') as file: 
            pickle.dump(self.model, file) 


    def load_edu_file(self, pkl_filename ='model.pkl'):
        pkl_filename = QFileDialog.getOpenFileName(directory='_Models', caption='Загрузите модель')[0]
        if pkl_filename == '':
            return
        with open(pkl_filename, 'rb') as file: 
            self.model = pickle.load(file)
            if self.model.get_name == 'Basic':
                self.rb_base.setChecked(True)
            if self.model.get_name == 'MLP':
                self.rb_mlp.setChecked(True)
            elif self.model.get_name == 'DTC':
                self.rb_dtc.setChecked(True)
            elif self.model.get_name == 'RFC':
                self.rb_rfc.setChecked(True)

        self.set_status(2)


    def do_model_eval(self):
        print('model eval')
        test_X = self.eval_list.drop(columns = ['File'])
        test_Y = pd.DataFrame({
            'fatique' : [1] * (len(self.data_list1) - len(self.edu_list1)) + [0] * (len(self.data_list2) - len(self.edu_list2))
        })
        y_pred = self.model.predict(test_X)
        m = accuracy_score(test_Y, y_pred)
        self.corr = m
        self.f1_line.setText(format(m,'.2f'))


    def do_eval(self):
        print('eval')
        #self.load_edu_file()
        self.res_list1 = ParsedData.get_df_null_row()
        self.res_list2 = ParsedData.get_df_null_row()
        file_data = self.eval_list.drop(columns='File')
        #, self.data_list2.drop(columns='File'), ignore_index=True)
        #file_data = self.data_list2.drop(columns='File')
    
        for col in self.keys.columns:
            if self.keys[col][0] == 0 and file_data.columns.__contains__(col):
                file_data = file_data.drop(columns=[col])
        tired = self.model.evaluate(file_data)
        for i in range(len(tired)):
            if not tired[i]:
                self.res_list1.loc[len(self.res_list1)] = self.eval_list['File'][i]
            else:
                self.res_list2.loc[len(self.res_list2)] = self.eval_list['File'][i]
        
        self.corr = 0
        self.err = 0
        print(self.res_list1)
        for _, row in self.res_list1.iterrows():
            if self.data_list_text1.toPlainText().__contains__(row['File']) : # gjghfdbnm
                self.corr += 1
            else:
                self.err += 1
        for _, row in self.res_list2.iterrows():
            if self.data_list_text2.toPlainText().__contains__(row['File']): #
                self.corr += 1
            else:
                self.err += 1
        #print('Corr: ' + str(self.corr) + '  Err: ' + str(self.err))
        corr_sum = self.corr / (self.corr + self.err)
        self.f1_line.setText(format(corr_sum, '.2f'))
        self.print_result()

   
    def print_result(self):
        print('print result')
        self.res_list_text1.setPlainText('')
        for _, row in self.res_list1.iterrows():
            self.res_list_text1.setPlainText(self.res_list_text1.toPlainText() + str(row['File']) + '\n')
        self.res_list_text2.setPlainText('')
        for _, row in self.res_list2.iterrows():
            self.res_list_text2.setPlainText(self.res_list_text2.toPlainText() + str(row['File']) + '\n')
        self.output_result()
        self.set_status(3)

    def output_result(self):
        df = pd.DataFrame({ 'File' : [], 'Eval' : [] })
        for _, row in self.res_list1.iterrows():
            df.loc[-1] = [row['File'], 1]
            df.index = df.index + 1
        for _, row in self.res_list2.iterrows():
            df.loc[-1] = [row['File'], 0]
            df.index = df.index + 1
        df.to_csv('output.csv', sep=';')



    def do_model_edu(self, redu=False):
        #self.edu_list1 = 
        train_X = pd.concat([self.edu_list1.drop(columns = ['File']), self.edu_list2.drop(columns = ['File'])])
        test_X = self.eval_list.drop(columns = ['File'])
        for col in self.keys.columns:
            if self.keys[col][0] == 0 and col in train_X.columns:
                train_X = train_X.drop(columns=[col])
                test_X = test_X.drop(columns=[col])
        
        if self.normalize_box.isChecked():
            gen = pd.concat([train_X, test_X], ignore_index=True)
            train_X = (train_X - gen.mean())/gen.std()
            test_X = (test_X - gen.mean())/gen.std()

        train_Y = pd.DataFrame({
            'fatique' : [1] * len(self.edu_list1) + [0] * len(self.edu_list2)
        })
        test_Y = pd.DataFrame({
            'fatique' : [1] * (len(self.data_list1) - len(self.edu_list1)) + [0] * (len(self.data_list2) - len(self.edu_list2))
        })

        if redu:
            self.model.redu(train_X, train_Y, test_X, test_Y)   
        else:
            if self.model_name == 'Basic':
                self.model = BasicEval()
            if self.model_name == 'RFC':
                self.model = RandomForestEval()
            elif self.model_name == 'DTC':
                self.model = DecisionTreeEval()
            elif self.model_name == 'MLP':
                self.model = MLPEval()
            if self.crossvalid_box.isChecked():
                data_X = pd.concat([self.data_list1.drop(columns = ['File']), self.data_list2.drop(columns = ['File'])], ignore_index=True)
                if self.normalize_box.isChecked():
                    data_X = (data_X - data_X.mean())/data_X.std()
                data_X.dropna()
                data_Y = pd.DataFrame({
                        'fatique' : [1] * len(self.data_list1) + [0] * len(self.data_list2)
                })
                data = pd.concat([data_X, data_Y], axis=1)
                data = data.sample(frac=1)
                data.to_csv('xx.csv', sep=';')
                data_Y = data['fatique']
                data_X = data.drop(columns=['fatique'])
                self.model.cross_edu(data_X, data_Y)
                self.f1_line.setText('F1: ' + format(self.model.f1,'.3f') + 
                                    ' Acc: ' + format(self.model.acc, '.3f'))
                self.crossvalid_label.setText('F1: ' + format(self.model.cross_f1,'.3f') + 
                                    ' Acc: ' + format(self.model.cross_acc, '.3f'))
            else:
                print(len(train_X), len(train_Y), len(test_X), len(test_Y))
                self.model.edu(train_X, train_Y, test_X, test_Y)
        #self.model = RandomForestEval.edu(train_X, train_Y, test_X, test_Y)

    def set_status(self, status):
        self.status = status
        self.status_text.setGeometry(0, self.height()-30, 3000, 30)
        self.reset_status_btn.setGeometry(self.width()-100, self.height()-30, 100, 30)
        self.status_img.setGeometry(0, self.height()-30, 3000, 30)

        '''
        for widget in self.data_group:
            widget.setEnabled(status == 0)
        for widget in self.edu_group:
            widget.setEnabled(status == 1)
        for widget in self.eval_group:
            widget.setEnabled(status > 1)
        for widget in self.res_group:
            widget.setEnabled(status == 3)
        '''

        if status == 0:
            self.status_text.setText('  Загрузите данные для обработки')
            self.status_img.setPixmap(QPixmap('UI/Red.png'))
        if status == 1:
            self.status_text.setText('  Обучите модель или загрузите уже обученную')
            self.status_img.setPixmap(QPixmap('UI/Orange.png'))
        if status == 2:
            self.status_text.setText('  Программа готова к запуску оценки')
            self.status_img.setPixmap(QPixmap('UI/Yellow.png'))
        if status == 3:
            self.status_text.setText('  Оценка выполнена')
            self.status_img.setPixmap(QPixmap('UI/Green.png'))


if __name__ == '__main__':
    app = QApplication(sys.argv)
    sleep(10)
    ex = EvalApp()
    sys.exit(app.exec_())





class TestEvent:
    def __init__(self, source, event_type='Undefined', comment=''):
        self.source = source
        self.event_type = event_type
        self.comment = comment

    def __str__(self):
        return self.event_type + ':' + self.comment + '\nSource:' + self.source


class TestEventsCollector:
    main = None

    def __init__(self):
        self.feedbacks = []
        self.warnings = []
        self.errors = []
        self.print_while_run = True
        TestEventsCollector.main = self

    def collect(self, event):
        if self.print_while_run:
            print(event)
        if event.event_type == 'Feedback':
            self.feedbacks.append(event)
        if event.event_type == 'Warning':
            self.warnings.append(event)
        if event.event_type == 'Error':
            self.errors.append(event)


class TestEventsEmitter:
    def __init__(self, source_name, default_type='', default_comment=''):
        self.source_name = source_name
        self.default_type = default_type
        self.default_comment = default_comment

    def emit_feedback(self, comment=''):
        TestEventsCollector.main.collect(TestEvent(self.source_name, 'Feedback', comment))

    def emit_warning(self, comment=''):
        TestEventsCollector.main.collect(TestEvent(self.source_name, 'Warning', comment))

    def emit_error(self, comment=''):
        TestEventsCollector.main.collect(TestEvent(self.source_name, 'Error', comment))

    def default_emit(self):
        TestEventsCollector.main.collect(TestEvent(self.source_name, self.default_type, self.default_comment))



# input init file
# модуль загрузки данных в систему
# сюда относится всё, связанное с считыванием и форматированием входных данных


from .InputSection import InputSection
from ..DeviceManager import DeviceHandler, Synchronizer


def read_video_file(path):
    # считывание файла
    return InputSection()


def read_csv_file(path):
    # считывание файла
    device = DeviceHandler(path)
    data = InputSection()
    data.add_data_array(device.request_data())
    return data

def init_reading_stream(path):
    # запуск потока ввода
    sync = Synchronizer(path)
    #sync.start_stream()



import math

# Фрагмент данных
class SingleData:
    def __init__(self):
        self.x = 0
        self.y = 0
        self.time = 0
    # инициализация с учётом угла обзора камеры сцены (задаётся в классе InputSection)
    def __init__(self, raw_data, fov):
        self.time = float(raw_data[0])
        self.x = float(raw_data[1]) * fov
        self.y = float(raw_data[2]) * fov
    # инициализация промежуточного кадра (исползуется для апроскимации )
    def approx(data1, data2, j, count): # j - номер кадра в промежутке, count - их обзщее количество
        sd = SingleData()
        sd.time = data1.time + (data2.time-data1.time) / count * j
        sd.x = data1.x + (data2.x-data1.x) / count * j
        sd.y = data1.y + (data2.y-data1.y) / count * j
        return sd
    # отображение в стороков формате
    def __str__(self):
        return self.time.__str__() + ': ' + self.x.__str__() + ' ; ' + self.y.__str__()
    # рассчёт дистаниции между двумя точками
    def get_distance(self, other):
        return math.sqrt(math.pow(self.x-other.x, 2) + math.pow(self.y-other.y, 2))



from time import sleep
import sys
from PyQt6.QtGui import QPainter, QColor

from ..Input import read_csv_file
from ..Analise.ParsedData import ParsedData
from PyQt6.QtWidgets import QMainWindow, QPushButton, QApplication, QLabel, QFileDialog, QLineEdit, QRadioButton, \
    QButtonGroup

class Example(QMainWindow):

    def __init__(self):
        super().__init__()

        self.rb_speed = QRadioButton('Speed', self)
        self.rb_speed.setChecked(True)
        self.I = 0
        self.rb_area = QRadioButton('Area', self)
        self.rb_dist = QRadioButton('Dist', self)

        self.rb_group = QButtonGroup()
        self.rb_group.addButton(self.rb_speed)
        self.rb_group.addButton(self.rb_area)
        self.rb_group.addButton(self.rb_dist)
        self.rb_group.buttonClicked.connect(self.rb_click)

        self.method_label = QLabel('Определение фиксаций\nпо скорости', self)
        self.method_label.adjustSize()
        self.file_line = QLineEdit(self)
        self.arg_label = QLabel("Порог скорости", self)
        self.arg_line = QLineEdit(self)

        self.arg_0_line = QLineEdit(self)
        self.arg_1_line = QLineEdit(self)
        self.arg_step_line = QLineEdit(self)
        """
        self.rad_line = QLineEdit(self)
        self.x0_line = QLineEdit(self)
        self.x1_line = QLineEdit(self)
        """
        self.initUI()

    def rb_click(self, rb):
        if rb.text() == 'Speed':
            self.I = 0
            self.method_label.setText('Определение фиксаций\nпо скорости')
            self.arg_line.setText('220')
            self.arg_label.setText('Порог скорости')
        if rb.text() == 'Area':
            self.I = 1
            self.method_label.setText('Определение фиксаций\nпо области')
            self.arg_line.setText('5')
            self.arg_label.setText('Диаметр области фиксации')
        if rb.text() == 'Dist':
            self.I = 2
            self.method_label.setText('Определение фиксаций\nпо абсолютному расстоянию')
            self.arg_line.setText('5')
            self.arg_label.setText('Максимальное расстояние')
        self.method_label.adjustSize()
        self.arg_label.adjustSize()

    def initUI(self):

        self.rb_speed.move(20, 60)
        self.rb_area.move(20, 80)
        self.rb_dist.move(20, 100)

        self.method_label.move(20, 20)


        # rad_label = QLabel('Диаметр центральной области', self)
        # rad_label.move(200, 20)
        # rad_label.adjustSize()
        # self.rad_line.setText('5')
        # self.rad_line.setGeometry(200, 40, 100, 20)
        self.arg_line.setGeometry(200, 80, 100, 20)
        self.arg_line.setText('220')
        self.arg_label.move(200, 60)
        self.arg_label.adjustSize()

        self.arg_0_line.setGeometry(20, 160, 30, 20)
        self.arg_0_line.setText('1')

        self.arg_1_line.setGeometry(50, 160, 30, 20)
        self.arg_1_line.setText('2.5')

        self.arg_step_line.setGeometry(80, 160, 30, 20)
        self.arg_step_line.setText('0.2')

        load_label = QLabel('Расчёт делается\nсразу по\nвыбору файла', self)
        load_label.move(380, 20)
        load_label.adjustSize()
        load_btn = QPushButton('Выбрать файлы', self)
        load_btn.setGeometry(380, 80, 100, 30)
        load_btn.clicked.connect(self.csvLoadButton)
        file_label = QLabel('Output file', self)
        file_label.move(380, 110)
        self.file_line.setText('masterfile.xlsx')
        self.file_line.setGeometry(380, 140, 100, 20)

        ppi_label = QLabel('Определение PPI', self)
        ppi_label.adjustSize()
        ppi_label.move(220, 140)
        load_btn = QPushButton('Выбрать файлы PPI', self)
        load_btn.setGeometry(220, 160, 120, 30)
        load_btn.clicked.connect(self.ppiLoadButton)

        find_btn = QPushButton('Счёт в \n интервале', self)
        find_btn.setGeometry(120, 145, 65, 40)
        find_btn.clicked.connect(self.findSec)
        """
        self.x0_line.setText('0')
        self.x0_line.setGeometry(20, 470, 40, 20)

        self.x1_line.setText('0')
        self.x1_line.setGeometry(440, 470, 40, 20)

        btn_left = QPushButton("<", self)
        btn_left.setGeometry(80, 460, 30, 30)
        btn_left.clicked.connect(self.btn_left_click)

        btn_right = QPushButton(">", self)
        btn_right.setGeometry(390, 460, 30, 30)
        btn_right.clicked.connect(self.btn_right_click)

        draw_btn = QPushButton("Draw", self)
        draw_btn.setGeometry(200, 460, 100, 30)
        draw_btn.clicked.connect(self.draw_graf)
        """
        pupil_btn = QPushButton('Файлы pupillabs', self)
        pupil_btn.setGeometry(480, 80, 100, 30)
        pupil_btn.clicked.connect(self.pupilButton)

        # pupil_c_btn = QPushButton('Корреляция pupillabs', self)
        # pupil_c_btn.setGeometry(480, 40, 100, 30)
        # pupil_c_btn.clicked.connect(self.pupilCoreButton)

        self.statusBar()

        self.setGeometry(300, 300, 600, 200)
        self.setWindowTitle('Вычисление количественных характеристик')
        self.show()

    def paintEvent(self, event):
        """
        qp = QPainter()
        qp.begin(self)
        self.draw_frame(qp)
        qp.end()
        """

    def draw_frame(self, qp):
        qp.setPen(QColor(168, 34, 3))
        qp.drawLine(10, 200, 490, 200)
        qp.drawLine(10, 450, 490, 450)

    def btn_left_click(self):
        self.x0_line.setText((float(self.x0_line.text()) - 10).__str__())
        self.x1_line.setText((float(self.x0_line.text()) - 10).__str__())

    def btn_right_click(self):
        self.x0_line.setText((float(self.x0_line.text()) + 10).__str__())
        self.x1_line.setText((float(self.x0_line.text()) + 10).__str__())

    def draw_graf(self):
        qp = QPainter()
        qp.begin(self)
        self.draw_frame(qp)

        file = QFileDialog.getOpenFileName()
        if not file.__contains__('.csv'):
            return
        section = read_csv_file(file)
        metrics = ParsedData()
        print('file chosen for drawing')
        args = [float(self.speed_line.text()), float(self.area_line.text())
            , float(self.s_dist_line.text())]
        metrics.parse(section, self.I, args[self.I])

        x0 = float(self.x0_line.text())
        x1 = float(self.x1_line.text())
        y0 = 0
        y1 = 2
        qp.setPen(QColor(0, 0, 0))
        for i in range(1, len(section.positionData)-1):
            qp.drawLine()
        qp.end()

    def ppiLoadButton(self):
        print('ppi button click')
        ppi_path, _ = QFileDialog.getOpenFileNames()
        if len(ppi_path) == 0:
            return
        print('files chosen')
        for file in ppi_path:
            if not file.__contains__('.txt'):
                print('cant read file ' + file)
                continue
            print('reading file ' + file)
            lines = open(file).readlines()
            ParsedData.ppi_to_xls(file, lines, self.file_line.text())

    def pupilButton(self):
        print('pupil button click')
        csv_path, _ = QFileDialog.getOpenFileNames()
        pupil_path, _ = QFileDialog.getOpenFileNames()

        if len(csv_path) == 0:
            return
        print('files chosen')
        if len(pupil_path) != len(csv_path):
            print('number of file are not matching')
            return
        for i in range(0, len(csv_path)):
            if not csv_path[i].__contains__('.csv'):
                print('cant read file ' + csv_path[i])
                continue
            if not pupil_path[i].__contains__('.csv'):
                print('cant read file ' + pupil_path[i])
                continue
            print('reading file ' + csv_path[i] + ' with data ' + pupil_path[i])
            section = read_csv_file(csv_path[i])
            metrics = ParsedData()
            metrics.parse_pupil(section, pupil_path[i])
            metrics.calc_metrics()
            metrics.to_xls(csv_path[i].split('/')[-1], self.file_line.text())

    def pupilCoreButton(self):
        print('pupil button click')
        csv_path, _ = QFileDialog.getOpenFileName()
        pupil_path, _ = QFileDialog.getOpenFileName()

        print('files chosen')
        if not csv_path.__contains__('.csv'):
            print('cant read file ' + csv_path)
            return
        if not pupil_path.__contains__('.csv'):
            print('cant read file ' + pupil_path)
            return
        print('reading file ' + csv_path + ' with data ' + pupil_path)
        section = read_csv_file(csv_path)
        pupil_metrics = ParsedData()
        pupil_metrics.parse_pupil(section, pupil_path)
        max_comp = 0
        max_i = 0
        p_list = []
        step = 0.1
        for i in range(0, 20):
            metrics = ParsedData()
            metrics.parse(section, 2, 0.1 + i * step, 1)
            comp = metrics.compare(pupil_metrics)
            p_list.append([comp, 0.1 + i * step])
            if max_comp < comp:
                max_comp = comp
                max_i = i
        print(p_list)
        print("Comp: " + max_comp.__str__() + " i " + (0.1 + max_i * step).__str__())


    def csvLoadButton(self):
        print('load button click')
        csv_path, _ = QFileDialog.getOpenFileNames()
        if len(csv_path) == 0:
            return
        print('files chosen')
        arg = float(self.arg_line.text())
        # rad = float(self.rad_line.text()) / 2
        for file in csv_path:
            if not file.__contains__('.csv'):
                print('cant read file ' + file)
                continue
            print('reading file ' + file)
            section = read_csv_file(file)
            print('parsing file by ' + self.arg_label.text() + ' = ' + self.arg_line.text())
            metrics = ParsedData()
            metrics.parse(section, self.I, arg)
            metrics.calc_metrics()
            metrics.to_csv(file.split('/')[-1], self.file_line.text())
    #        metrics.print_some()
    #        metrics.to_csv('../out_data1 — копия.xlsx')

    def findSec(self):
        print('find sec click')
        csv_path, _ = QFileDialog.getOpenFileNames()
        if len(csv_path) == 0:
            return
        print('files chosen')
        arg0 = float(self.arg_0_line.text())
        arg1 = float(self.arg_1_line.text())
        step = float(self.arg_step_line.text())
        # rad = float(self.rad_line.text()) / 2
#        diff_list = []

        total = len(csv_path) * ((arg1 - arg0)/step).__int__()
        current = 0
        for ii in range(0, ((arg1 - arg0)/step).__int__()):
            arg = arg0 + ii * step
#            lines = []
            for file in csv_path:
                if not file.__contains__('.csv'):
                    print('cant read file ' + file)
                    continue
                print('reading file ' + file)
                section = read_csv_file(file)
                metrics = ParsedData()
                metrics.parse(section, self.I, arg)
                metrics.calc_metrics()
#                metrics.to_xls_by_row(file.split('/')[-1], self.file_line.text())
                metrics.to_csv(file.split('/')[-1], 'Summary.csv')
#                lines.append(metrics.get_line())
                current += 1
                print( (float(current) / float(total) * 100.0).__str__() + " % of progress")
            '''
                print('calculating differences...')
                diff = [0.0] * len(lines[0])
                for i in range(1, len(lines)):
                    for j in range(0, len(lines[0])):
                        if (lines[-1][j] - lines[0][j] < 0) == (lines[0][j] - lines[i][j] < 0):
                            diff[j] += 1
                fl = False
                for j in range(0, len(lines[0])):
                    diff[j] = diff[j] * 100 / (len(lines) - 1)
                    fl = fl or diff[j] > 85
                if fl:
                    diff_list.append(arg.__str__() + ' : ' + diff.__str__())
            for line in diff_list:
                print(line)
            '''

    def buttonAClicked(self):
        self.statusBar().showMessage('A was pressed' + self.line.text())

    def buttonBClicked(self):
        self.statusBar().showMessage('B was pressed' + self.line.text())

if __name__ == '__main__':
    app = QApplication(sys.argv)
    sleep(10)
    ex = Example()
    sys.exit(app.exec_())



from setuptools import find_packages, setup


def get_long_description():
    return 'https://github.com/AI-group-72/FAEyeTON/blob/main/README.md'

def get_requirements():
    return ['pandas', 'openpyxl', 'scikit-learn', 'numpy', 'PyQt6']

setup(
    name="EyeTrackFatigue",
    version="1.0.6",
    description="проект FAEyeTON - Библиотека открытого кода для оценки функционального состояния утомления оператора на основе динамической активности взгляда и головы. Проект Код-ИИ-2022.",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Кашевник А.М. Кузнецов В.В. Брак И.В. Мамонов А.А. Коваленко С.Д.",
    author_email="anton.mamonov.golohvastogo@mail.ru",
    url="https://github.com/AI-group-72/FAEyeTON",
    project_urls={
        "GitHub Project": "https://github.com/AI-group-72/FAEyeTON",
        "Issue Tracker": "https://github.com/AI-group-72/FAEyeTON/Meta/issues",
    },
    packages=['EyeTrackFatigue.DeviceManager', 'EyeTrackFatigue.Input',
              'EyeTrackFatigue.Analise', 'EyeTrackFatigue.Evaluate', 'EyeTrackFatigue.UI', 'EyeTrackFatigue.Meta'],
    install_requires=get_requirements(),
    keywords=[
        "EyeTrackFatigue",
    ],
    license="GNU LGPL",
)




import os
import numpy as np
import cv2
import glob
from sys import argv
from math import pi
import datetime
import os.path


def HashString(s):
    result = ''
    for i in range(len(s)):
        if ord('0') <= ord(s[i]) <= ord('9'):
            result = result + s[i]
    return result


def DoLog(line):
    # запись лога
    path = os.curdir + '\\log-' + HashString(argv[1]) + '.txt'
    file1 = open(path, 'a')
    line = str(datetime.datetime.now()) + ' ' + line + '\n'
    file1.write(line)
    file1.close()


def LastLine(path):
    with open(path) as f:
        for line in f:
            pass
    return line
    f.close()


def DoProcessing(inputVideo):
    # на вход получает путь к видеофайлу.
    # вычисляется относительное (!) смещение головы в градусах и наклон в градусах.
    # результат записывается в текстовый файл и в массив transforms.
    EyetrackerFOV = 82 # указание угла обзора камеры сцены окулографа, в угловых градусах
    EyetrackerPixels = 1084 # указание расширения окулографа, в пикселях
    EyetrackerPixToFOV = EyetrackerFOV/EyetrackerPixels  # для перевода смещения из пикселей в градусы для камеры сцены 

    outputTxt = inputVideo + '+alpha.csv'
    DoLog('start processing of ' + inputVideo)
    cap = cv2.VideoCapture(inputVideo)
    num_frames = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))

    if os.path.isfile(outputTxt):
        # проверяем, вдруг мы этот файл уже обработали, полностью или частично
        line = LastLine(outputTxt)
        prevFrameNo = int(line.split(';')[0]) - 1

        if (num_frames - prevFrameNo) < 10:
            # сброс хвоста видео
            DoLog('already processed ' + inputVideo)
            print('already processed ' + inputVideo)
            return
        print('Skiping frames. Start processing from frame No ' + str(prevFrameNo))
        for i in range(0, prevFrameNo):
            _, prev_frame = cap.read()
            if i/1000 == round(i/1000):
                print(i)
        fileTxt = open(outputTxt, 'a')
        prevFrameNo = prevFrameNo
    else:
        # файл ни разу не обрабатывался ранее
        prevFrameNo = 0
        fileTxt = open(outputTxt, 'w')
        fileTxt.write('FrameNo;eye_X;eye_Y;FlightInstr1_X;FlightInstr1_Y;FlightInstr1_W;FlightInstr1_H;FlightInstr2_X;FlightInstr2_Y;FlightInstr2_W;FlightInstr2_H;FlightInstr3_X;FlightInstr3_Y;FlightInstr3_W;FlightInstr3_H;FlightInstr4_X;FlightInstr4_Y;FlightInstr4_W;FlightInstr4_H;FrameShift_X;frameShift_Y;frameRotation_deg\n')
        fileTxt.write('1;-0;-0;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;0;0;0;\n')

    _, prev_frame = cap.read()
    prev_gray = cv2.cvtColor(prev_frame, cv2.COLOR_BGR2GRAY)
    transforms = np.zeros((num_frames - 1, 3), np.float32)

    for i in range(prevFrameNo, num_frames - 1):
        if i == round(num_frames * 0.01):
            print(inputVideo + ":  1% of processing done")
        if i == round(num_frames * 0.33):
            print(inputVideo + ": 33% of processing done")
        if i == round(num_frames * 0.66):
            print(inputVideo + ": 66% of processing done")

        # Расчёт оптического потока между кадрами / Calculate optical flow between consecutive frames
        translation_x = 0
        translation_y = 0
        rotation_angle = 0

        prev_points = cv2.goodFeaturesToTrack(
            prev_gray,
            maxCorners=200,
            qualityLevel=0.1,
            minDistance=20,
            blockSize=4
        )
        success, curr_frame = cap.read()
        if not success:
            DoLog('Error reading file '+inputVideo + ' |frame ' + str(i))
            break

        curr_gray = cv2.cvtColor(curr_frame, cv2.COLOR_BGR2GRAY)
        try:
            # вычисляем оптический поток при помощи Канаде-Лукаса
            curr_points, status, err = cv2.calcOpticalFlowPyrLK(
                prev_gray,
                curr_gray,
                prev_points,
                None,
                2
            )
        except:
            # обработка битых частей видео, где кадр представляет собой, например, серый прямоугольник
            DoLog('another bad frame ' + str(i+2) + ' in ' + inputVideo)
            line = str(i+2) + ';-0;-0;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;' + str(translation_x) + ';' + str(translation_y) + ';' + str(rotation_angle) + ';\n'
            fileTxt.write(line)
            # prev_gray = curr_gray
            continue

        assert prev_points.shape == curr_points.shape
        idx = np.where(status == 1)[0]
        prev_points = prev_points[idx]
        curr_points = curr_points[idx]

        if len(curr_points) > 0:
            # Оценка аффинных преобразований (включая поворот) между точками / Estimate affine transformation between the points
            try:
                matrix, _ = cv2.estimateAffine2D(prev_points, curr_points)
                translation_x = matrix[0, 2]
                translation_y = matrix[1, 2]
                rotation_angle = (180/pi) * np.arctan2(matrix[1, 0], matrix[0, 0])

                # переводим из пикселей в градусы
                translation_x = translation_x * EyetrackerPixToFOV
                translation_y = translation_y * EyetrackerPixToFOV
            except:
                DoLog('bad frame ' + str(i+2) + ' in ' + inputVideo)

        if abs(rotation_angle) > 15:
            # отбрасываем невозможно резкие наклоны головы, явный артефакт записи
            translation_x = 0
            translation_y = 0
            rotation_angle = 0

        transforms[i] = [translation_x, translation_y, rotation_angle]

        translation_x = round(translation_x, 3)
        translation_y = round(translation_y, 3)
        rotation_angle = round(rotation_angle, 3)
        line = str(translation_x) + ';' + str(translation_y) + ';' + str(rotation_angle) + ';\n'
        line = str(i+2) + ';-0;-0;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;-1;' + line
        fileTxt.write(line)
        prev_gray = curr_gray

    fileTxt.close()
    cap.release()
    DoLog('finished processing of ' + inputVideo)
    # return transforms


if __name__ == '__main__':
    # скрипт принимает 1 аргумент командной строки - маску для поиска видеофайлов.
    # например:
    # c:\Folder\Experiments\Participant_5\*\Eyes\*\*.mp4
    # обработает все файлы по удовлетворяющем маску путям.

    print('mask: ', argv[1])
    mask = argv[1]
    mp4list = glob.glob(mask, recursive=True)
    print('\nfiles found:\n')
    print(*mp4list, sep='\n')
    print('Starting to process ' + str(len(mp4list)) + ' files')
    for i in range(len(mp4list)):
        print('processing ' + mp4list[i])
        DoProcessing(mp4list[i])
        print('File is 100% processed.\n')



import datetime
import pandas as pd
import joblib
from PyQt6.QtWidgets import *
from sklearn.metrics import accuracy_score, f1_score
from Input import read_csv_file
from Analise.ParsedData import ParsedData
now1 = datetime.datetime.now()
print(now1)

section = read_csv_file('Samples/five minutes short.csv')
metrics = ParsedData()
metrics.parse(section, 1, 1.0)
metrics.calc_metrics()

with open('Samples/MLP2.sav', 'rb') as file: 
    model = joblib.load(file)


chosen = ['x_mean', 'x_std', 'x_min', 'x_max', 'x_25', 'x_50', 'x_75', 'y_mean', 'y_std', 'y_min', 'y_max', 'y_25', 'y_50', 'y_75'] #, 'Saccades with amplitude < 6 degrees, per minute', 'Max Curve', 'Fixation time > 150 ms', 'Fixation time > 180 ms', '% of Fixations < 150 ms', '% of Fixations > 150 ms', '% of Fixations > 900 ms', '% of Fixations < 180 ms', '% of Fixations > 180 ms', 'Fixation time < 150 ms, per time', 'Fixation time > 150 ms, per time', '% of Fixations > 150 ms, per minute', '% of Fixations < 180 ms, per minute', 'Min Speed', 'Max Speed', 'Average Speed in interval (1s)', 'Max Speed in interval (1s)', 'Average Fixation Speed', 'Max Fixation Speed', 'Average Saccade Length', 'Min Saccade Length', 'Max Saccade Length']
data_row = metrics.get_df_row()[chosen]

predict_Y = model.evaluate(data_row)
print('Accuracy Score:', accuracy_score([1], predict_Y))
print('F1 Score:', f1_score([1], predict_Y))

now = datetime.datetime.now()
print(now)
print('Total time:', now - now1)




from ..Evaluate.Evaluator import Evaluator
import numpy as np
class BasicEval(Evaluator):
    def __init__(self):
        self.tired = []
        self.fresh = []
        self.corrs = []
    
    def get_name(self):
        return 'Basic'

    def edu(self, train_X, train_Y, test_X, test_Y):
        self.tired = [0] * len(train_X.head(1))
        self.fresh = [0] * len(train_X.head(1))
        tired_count = 0
        fresh_count = 0
        self.corrs = []
        for col in train_X.columns:
            Sum_xy = sum((train_X[col]-train_X[col].mean())*(train_Y[train_Y.columns[0]]-train_Y[train_Y.columns[0]].mean()))
            Sum_x_squared = sum((train_X[col]-train_X[col].mean())**2)
            Sum_y_squared = sum((train_Y[train_Y.columns[0]]-train_Y[train_Y.columns[0]].mean())**2)       
            self.corrs.append(Sum_xy / np.sqrt(Sum_x_squared * Sum_y_squared))
        '''
        for i, row in train_X.iterrows():
            if train_Y.iloc[i][0] == 0:
                tired_count += 1
                self.tired += row.tolist()
            else:
                fresh_count += 1
                self.fresh += row.tolist()
        for i in range(len(self.tired)):
            self.tired[i] /= tired_count
            self.fresh[i] /= fresh_count
        '''

    def evaluate(self, data):
        pred_Y = []
        for _, row in data.iterrows():
            fl = 0
            for i in range(len(row)):
                fl += row[i] * self.corrs[i]
                #fl += 1 if abs(row[i] - self.tired[i]) < abs(row[i] - self.fresh[i]) else -1
            pred_Y.append(0 if fl < 0 else 1)
        return pred_Y
            
